@@ -52,6 +52,8 @@ class QAudioProbe;
 class QAudioBuffer;
 QT_END_NAMESPACE
 
+class QAudioLevel;
+
 class AudioRecorder : public QMainWindow
 {
     Q_OBJECT
@@ -68,16 +70,18 @@ private slots:
     void togglePause();
     void toggleRecord();
 
-    void updateState(QMediaRecorder::State);
+    void updateStatus(QMediaRecorder::Status);
     void updateProgress(qint64 pos);
     void displayErrorMessage();
-    QUrl generateAudioFilePath();
 
 private:
+    void clearAudioLevels();
+
     Ui::AudioRecorder *ui;
 
     QAudioRecorder *audioRecorder;
     QAudioProbe *probe;
+    QList<QAudioLevel*> audioLevels;
     bool outputLocationSet;
 
 };
