@@ -72,7 +72,11 @@ bool QGstAppSrc::setup(GstElement* appsrc)
     if (m_setup || m_stream == 0 || appsrc == 0)
         return false;
 
+    if (m_appSrc)
+        gst_object_unref(G_OBJECT(m_appSrc));
+
     m_appSrc = GST_APP_SRC(appsrc);
+    gst_object_ref(G_OBJECT(m_appSrc));
     gst_app_src_set_callbacks(m_appSrc, (GstAppSrcCallbacks*)&m_callbacks, this, (GDestroyNotify)&QGstAppSrc::destroy_notify);
 
     g_object_get(G_OBJECT(m_appSrc), "max-bytes", &m_maxBytes, NULL);
@@ -149,28 +153,29 @@ void QGstAppSrc::pushDataToAppSrc()
             size = qMin(m_stream->bytesAvailable(), queueSize());
         else
             size = qMin(m_stream->bytesAvailable(), (qint64)m_dataRequestSize);
-        void *data = g_malloc(size);
-        GstBuffer* buffer = gst_app_buffer_new(data, size, g_free, data);
-        buffer->offset = m_stream->pos();
-        qint64 bytesRead = m_stream->read((char*)GST_BUFFER_DATA(buffer), size);
-        buffer->offset_end =  buffer->offset + bytesRead - 1;
 
-        if (bytesRead > 0) {
-            m_dataRequested = false;
-            m_enoughData = false;
-            GstFlowReturn ret = gst_app_src_push_buffer (GST_APP_SRC (element()), buffer);
-            if (ret == GST_FLOW_ERROR) {
-                qWarning()<<"appsrc: push buffer error";
-            } else if (ret == GST_FLOW_WRONG_STATE) {
-                qWarning()<<"appsrc: push buffer wrong state";
-            } else if (ret == GST_FLOW_RESEND) {
-                qWarning()<<"appsrc: push buffer resend";
+        if (size) {
+            void *data = g_malloc(size);
+            GstBuffer* buffer = gst_app_buffer_new(data, size, g_free, data);
+            buffer->offset = m_stream->pos();
+            qint64 bytesRead = m_stream->read((char*)GST_BUFFER_DATA(buffer), size);
+            buffer->offset_end =  buffer->offset + bytesRead - 1;
+
+            if (bytesRead > 0) {
+                m_dataRequested = false;
+                m_enoughData = false;
+                GstFlowReturn ret = gst_app_src_push_buffer (GST_APP_SRC (element()), buffer);
+                if (ret == GST_FLOW_ERROR) {
+                    qWarning()<<"appsrc: push buffer error";
+                } else if (ret == GST_FLOW_WRONG_STATE) {
+                    qWarning()<<"appsrc: push buffer wrong state";
+                } else if (ret == GST_FLOW_RESEND) {
+                    qWarning()<<"appsrc: push buffer resend";
+                }
             }
-        }
-
-        // After reading we might be all done
-        if (m_stream->atEnd())
+        } else {
             sendEOS();
+        }
     } else if (m_stream->atEnd()) {
         sendEOS();
     }
