@@ -62,8 +62,6 @@
 #include <qaudioformat.h>
 
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QIODevice;
@@ -165,6 +163,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QSAMPLECACHE_P_H
