@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the Qt Toolkit.
@@ -147,9 +147,20 @@ public:
 
 Q_SIGNALS:
     void contextReady();
+    void contextFailed();
     void volumeChanged();
 
-private:
+private Q_SLOTS:
+    void onContextFailed()
+    {
+        release();
+
+        // Try to reconnect later
+        QTimer::singleShot(30000, this, SLOT(prepare()));
+
+        emit contextFailed();
+    }
+
     void prepare()
     {
         m_vol = PA_VOLUME_NORM;
@@ -172,16 +183,17 @@ private:
         lock();
         m_context = pa_context_new(m_mainLoopApi, QString(QLatin1String("QtPulseAudio:%1")).arg(::getpid()).toLatin1().constData());
 
-        pa_context_set_state_callback(m_context, context_state_callback, this);
-
         if (m_context == 0) {
             qWarning("PulseAudioService: Unable to create new pulseaudio context");
             pa_threaded_mainloop_unlock(m_mainLoop);
             pa_threaded_mainloop_free(m_mainLoop);
             m_mainLoop = 0;
+            onContextFailed();
             return;
         }
 
+        pa_context_set_state_callback(m_context, context_state_callback, this);
+
         if (pa_context_connect(m_context, 0, (pa_context_flags_t)0, 0) < 0) {
             qWarning("PulseAudioService: pa_context_connect() failed");
             pa_context_unref(m_context);
@@ -196,12 +208,23 @@ private:
         m_prepared = true;
     }
 
+private:
     void release()
     {
-        if (!m_prepared) return;
-        pa_context_unref(m_context);
-        pa_threaded_mainloop_stop(m_mainLoop);
-        pa_threaded_mainloop_free(m_mainLoop);
+        if (!m_prepared)
+            return;
+
+        if (m_context) {
+            pa_context_unref(m_context);
+            m_context = 0;
+        }
+
+        if (m_mainLoop) {
+            pa_threaded_mainloop_stop(m_mainLoop);
+            pa_threaded_mainloop_free(m_mainLoop);
+            m_mainLoop = 0;
+        }
+
         m_prepared = false;
     }
 
@@ -221,6 +244,9 @@ private:
     #endif
                 QMetaObject::invokeMethod(self, "contextReady", Qt::QueuedConnection);
                 break;
+            case PA_CONTEXT_FAILED:
+                QMetaObject::invokeMethod(self, "onContextFailed", Qt::QueuedConnection);
+                break;
             default:
                 break;
         }
@@ -357,7 +383,7 @@ QSoundEffectPrivate::QSoundEffectPrivate(QObject* parent):
     m_muted(false),
     m_playQueued(false),
     m_stopping(false),
-    m_volume(100),
+    m_volume(1.0),
     m_loopCount(1),
     m_runningCount(0),
     m_reloadCategory(false),
@@ -490,14 +516,16 @@ void QSoundEffectPrivate::setLoopCount(int loopCount)
     if (loopCount == 0)
         loopCount = 1;
     m_loopCount = loopCount;
+    if (m_playing)
+        setLoopsRemaining(loopCount);
 }
 
-int QSoundEffectPrivate::volume() const
+qreal QSoundEffectPrivate::volume() const
 {
     return m_volume;
 }
 
-void QSoundEffectPrivate::setVolume(int volume)
+void QSoundEffectPrivate::setVolume(qreal volume)
 {
     m_volume = volume;
     emit volumeChanged();
@@ -511,7 +539,8 @@ void QSoundEffectPrivate::updateVolume()
     PulseDaemonLocker locker;
     pa_cvolume volume;
     volume.channels = m_pulseSpec.channels;
-    pa_operation_unref(pa_context_set_sink_input_volume(pulseDaemon()->context(), m_sinkInputId, pulseDaemon()->calcVolume(&volume, m_volume), setvolume_callback, m_ref->getRef()));
+    if (pulseDaemon()->context())
+        pa_operation_unref(pa_context_set_sink_input_volume(pulseDaemon()->context(), m_sinkInputId, pulseDaemon()->calcVolume(&volume, qRound(m_volume * 100)), setvolume_callback, m_ref->getRef()));
     Q_ASSERT(pa_cvolume_valid(&volume));
 #ifdef QT_PA_DEBUG
     qDebug() << this << "updateVolume =" << pa_cvolume_max(&volume);
@@ -535,7 +564,8 @@ void QSoundEffectPrivate::updateMuted()
     if (m_sinkInputId < 0)
         return;
     PulseDaemonLocker locker;
-    pa_operation_unref(pa_context_set_sink_input_mute(pulseDaemon()->context(), m_sinkInputId, m_muted, setmuted_callback, m_ref->getRef()));
+    if (pulseDaemon()->context())
+        pa_operation_unref(pa_context_set_sink_input_mute(pulseDaemon()->context(), m_sinkInputId, m_muted, setmuted_callback, m_ref->getRef()));
 #ifdef QT_PA_DEBUG
     qDebug() << this << "updateMuted = " << m_muted;
 #endif
@@ -619,7 +649,7 @@ void QSoundEffectPrivate::play()
             emptyStream();
             return;
         }
-        m_runningCount = m_loopCount;
+        setLoopsRemaining(m_loopCount);
         playSample();
     }
 
@@ -705,7 +735,7 @@ void QSoundEffectPrivate::sampleReady()
         }
 #endif
     } else {
-        if (pa_context_get_state(pulseDaemon()->context()) != PA_CONTEXT_READY) {
+        if (!pulseDaemon()->context() || pa_context_get_state(pulseDaemon()->context()) != PA_CONTEXT_READY) {
             connect(pulseDaemon(), SIGNAL(contextReady()), SLOT(contextReady()));
             return;
         }
@@ -741,6 +771,7 @@ void QSoundEffectPrivate::unloadPulseStream()
         pa_stream_disconnect(m_pulseStream);
         pa_stream_unref(m_pulseStream);
         disconnect(pulseDaemon(), SIGNAL(volumeChanged()), this, SLOT(updateVolume()));
+        disconnect(pulseDaemon(), SIGNAL(contextFailed()), this, SLOT(contextFailed()));
         m_pulseStream = 0;
         m_reloadCategory = false; // category will be reloaded when we connect anyway
     }
@@ -895,17 +926,17 @@ void QSoundEffectPrivate::createPulseStream()
     qDebug() << this << "createPulseStream";
 #endif
 
+    if (!pulseDaemon()->context())
+        return;
+
     pa_proplist *propList = pa_proplist_new();
-    if (m_category.isNull()) {
-        // Meant to be one of the strings "video", "music", "game", "event", "phone", "animation", "production", "a11y", "test"
-        pa_proplist_sets(propList, PA_PROP_MEDIA_ROLE, "game");
-    } else {
+    if (!m_category.isNull())
         pa_proplist_sets(propList, PA_PROP_MEDIA_ROLE, m_category.toLatin1().constData());
-    }
     pa_stream *stream = pa_stream_new_with_proplist(pulseDaemon()->context(), m_name.constData(), &m_pulseSpec, 0, propList);
     pa_proplist_free(propList);
 
     connect(pulseDaemon(), SIGNAL(volumeChanged()), this, SLOT(updateVolume()));
+    connect(pulseDaemon(), SIGNAL(contextFailed()), this, SLOT(contextFailed()));
 
     if (stream == 0) {
         qWarning("QSoundEffect(pulseaudio): Failed to create stream");
@@ -947,6 +978,12 @@ void QSoundEffectPrivate::contextReady()
     createPulseStream();
 }
 
+void QSoundEffectPrivate::contextFailed()
+{
+    unloadPulseStream();
+    connect(pulseDaemon(), SIGNAL(contextReady()), this, SLOT(contextReady()));
+}
+
 void QSoundEffectPrivate::stream_write_callback(pa_stream *s, size_t length, void *userdata)
 {
     Q_UNUSED(length);
