@@ -45,8 +45,6 @@
 #include <QtMultimedia/qmediaobject.h>
 #include <QtMultimedia/qmediaenumdebug.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -161,6 +159,7 @@ Q_SIGNALS:
 
 private:
     friend class QCamera;
+    friend class QCameraPrivate;
     explicit QCameraExposure(QCamera *parent = 0);
     virtual ~QCameraExposure();
 
@@ -183,6 +182,4 @@ Q_MEDIA_ENUM_DEBUG(QCameraExposure, ExposureMode)
 Q_MEDIA_ENUM_DEBUG(QCameraExposure, FlashMode)
 Q_MEDIA_ENUM_DEBUG(QCameraExposure, MeteringMode)
 
-QT_END_HEADER
-
 #endif // QCAMERAEXPOSURE_H
