@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the Qt Toolkit.
@@ -55,19 +55,15 @@
 
 QT_BEGIN_NAMESPACE
 
-namespace
+static void qRegisterCameraFocusMetaTypes()
 {
-class CameraFocusRegisterMetaTypes
-{
-public:
-    CameraFocusRegisterMetaTypes()
-    {
-        qRegisterMetaType<QCameraFocus::FocusModes>("QCameraFocus::FocusModes");
-        qRegisterMetaType<QCameraFocus::FocusPointMode>("QCameraFocus::FocusPointMode");
-    }
-} _registerCameraFocusMetaTypes;
+    qRegisterMetaType<QCameraFocus::FocusModes>("QCameraFocus::FocusModes");
+    qRegisterMetaType<QCameraFocus::FocusPointMode>("QCameraFocus::FocusPointMode");
 }
 
+Q_CONSTRUCTOR_FUNCTION(qRegisterCameraFocusMetaTypes)
+
+
 class QCameraFocusFakeZoomControl : public QCameraZoomControl
 {
 public:
