@@ -83,20 +83,15 @@ QT_BEGIN_NAMESPACE
     \value CaptureToBuffer  Capture the image to a buffer for further processing.
 */
 
-namespace
+static void qRegisterCameraImageCaptureMetaTypes()
 {
-class MediaRecorderRegisterMetaTypes
-{
-public:
-    MediaRecorderRegisterMetaTypes()
-    {
-        qRegisterMetaType<QCameraImageCapture::Error>("QCameraImageCapture::Error");
-        qRegisterMetaType<QCameraImageCapture::CaptureDestination>("QCameraImageCapture::CaptureDestination");
-        qRegisterMetaType<QCameraImageCapture::CaptureDestinations>("QCameraImageCapture::CaptureDestinations");
-    }
-} _registerRecorderMetaTypes;
+    qRegisterMetaType<QCameraImageCapture::Error>("QCameraImageCapture::Error");
+    qRegisterMetaType<QCameraImageCapture::CaptureDestination>("QCameraImageCapture::CaptureDestination");
+    qRegisterMetaType<QCameraImageCapture::CaptureDestinations>("QCameraImageCapture::CaptureDestinations");
 }
 
+Q_CONSTRUCTOR_FUNCTION(qRegisterCameraImageCaptureMetaTypes)
+
 
 class QCameraImageCapturePrivate
 {
@@ -266,8 +261,8 @@ bool QCameraImageCapture::setMediaObject(QMediaObject *mediaObject)
                         this, SIGNAL(imageMetadataAvailable(int,QString,QVariant)));
                 connect(d->control, SIGNAL(imageAvailable(int,QVideoFrame)),
                         this, SIGNAL(imageAvailable(int,QVideoFrame)));
-                connect(d->control, SIGNAL(imageSaved(int, QString)),
-                        this, SIGNAL(imageSaved(int, QString)));
+                connect(d->control, SIGNAL(imageSaved(int,QString)),
+                        this, SIGNAL(imageSaved(int,QString)));
                 connect(d->control, SIGNAL(readyForCaptureChanged(bool)),
                         this, SLOT(_q_readyChanged(bool)));
                 connect(d->control, SIGNAL(error(int,int,QString)),
