@@ -47,8 +47,6 @@
 
 #include <QtMultimedia/qcamerafocus.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 // Required for QDoc workaround
@@ -89,8 +87,6 @@ Q_MEDIA_DECLARE_CONTROL(QCameraFocusControl, QCameraFocusControl_iid)
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 
 #endif  // QCAMERAFOCUSCONTROL_H
 
