@@ -45,8 +45,6 @@
 #include <QtMultimedia/qmediacontrol.h>
 #include <QtMultimedia/qcameraimagecapture.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QImage;
@@ -92,8 +90,6 @@ Q_MEDIA_DECLARE_CONTROL(QCameraImageCaptureControl, QCameraImageCaptureControl_i
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 
 #endif  // QCAMERAIMAGECAPTURECONTROL_H
 
