@@ -48,8 +48,6 @@
 #include <QtMultimedia/qcamera.h>
 #include <QtMultimedia/qmediaenumdebug.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 // Required for QDoc workaround
@@ -97,7 +95,5 @@ Q_DECLARE_METATYPE(QCameraImageProcessingControl::ProcessingParameter)
 
 Q_MEDIA_ENUM_DEBUG(QCameraImageProcessingControl, ProcessingParameter)
 
-QT_END_HEADER
-
 #endif
 
