@@ -47,8 +47,6 @@
 
 #include <QtMultimedia/qcamera.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 // Required for QDoc workaround
@@ -59,7 +57,7 @@ class Q_MULTIMEDIA_EXPORT QCameraLocksControl : public QMediaControl
     Q_OBJECT
 public:
     ~QCameraLocksControl();
-    
+
     virtual QCamera::LockTypes supportedLocks() const = 0;
 
     virtual QCamera::LockStatus lockStatus(QCamera::LockType lock) const = 0;
@@ -79,8 +77,6 @@ Q_MEDIA_DECLARE_CONTROL(QCameraLocksControl, QCameraLocksControl_iid)
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 
 #endif  // QCAMERALOCKSCONTROL_H
 
