@@ -100,7 +100,7 @@ public:
                     m_extraInfo.clear();
                     int artistStart = line.indexOf(QLatin1String(","), 8);
                     bool ok = false;
-                    int length = line.mid(8, artistStart < 8 ? -1 : artistStart - 8).trimmed().toInt(&ok);
+                    int length = line.midRef(8, artistStart < 8 ? -1 : artistStart - 8).trimmed().toInt(&ok);
                     if (ok && length > 0) {
                         //convert from second to milisecond
                         m_extraInfo[QMediaMetaData::Duration] = QVariant(length * 1000);
@@ -108,13 +108,13 @@ public:
                     if (artistStart > 0) {
                         int titleStart = getSplitIndex(line, artistStart);
                         if (titleStart > artistStart) {
-                            m_extraInfo[QMediaMetaData::Author] = line.mid(artistStart + 1,
-                                                             titleStart - artistStart - 1).trimmed().
+                            m_extraInfo[QMediaMetaData::Author] = line.midRef(artistStart + 1,
+                                                             titleStart - artistStart - 1).trimmed().toString().
                                                              replace(QLatin1String("--"), QLatin1String("-"));
-                            m_extraInfo[QMediaMetaData::Title] = line.mid(titleStart + 1).trimmed().
+                            m_extraInfo[QMediaMetaData::Title] = line.midRef(titleStart + 1).trimmed().toString().
                                                    replace(QLatin1String("--"), QLatin1String("-"));
                         } else {
-                            m_extraInfo[QMediaMetaData::Title] = line.mid(artistStart + 1).trimmed().
+                            m_extraInfo[QMediaMetaData::Title] = line.midRef(artistStart + 1).trimmed().toString().
                                                    replace(QLatin1String("--"), QLatin1String("-"));
                         }
                     }
@@ -302,14 +302,14 @@ Version=2
             emit error(QPlaylistFileParser::FormatError, QString(tr("Error parsing playlist at line[%1]:%2")).arg(QString::number(lineIndex), line));
             return QString();
         }
-        return line.mid(start + 1).trimmed();
+        return line.midRef(start + 1).trimmed().toString();
     }
 
     void setCount(int count) {
         m_count = count;
-        m_fileName = QString(tr("File%1")).arg(count);
-        m_titleName = QString(tr("Title%1")).arg(count);
-        m_lengthName = QString(tr("Length%1")).arg(count);
+        m_fileName = QStringLiteral("File%1").arg(count);
+        m_titleName = QStringLiteral("Title%1").arg(count);
+        m_lengthName = QStringLiteral("Length%1").arg(count);
         m_item.clear();
         m_readFlags = 0;
     }
@@ -396,8 +396,8 @@ void QPlaylistFileParserPrivate::processLine(int startIndex, int length)
         Q_ASSERT(m_currentParser);
         connect(m_currentParser, SIGNAL(newItem(QVariant)), q, SIGNAL(newItem(QVariant)));
         connect(m_currentParser, SIGNAL(finished()), q, SLOT(_q_handleParserFinished()));
-        connect(m_currentParser, SIGNAL(error(QPlaylistFileParser::ParserError, QString)),
-                q, SLOT(_q_handleParserError(QPlaylistFileParser::ParserError, QString)));
+        connect(m_currentParser, SIGNAL(error(QPlaylistFileParser::ParserError,QString)),
+                q, SLOT(_q_handleParserError(QPlaylistFileParser::ParserError,QString)));
     }
 
     QString line;
@@ -583,8 +583,8 @@ void QPlaylistFileParser::stop()
     if (d->m_currentParser) {
         disconnect(d->m_currentParser, SIGNAL(newItem(QVariant)), this, SIGNAL(newItem(QVariant)));
         disconnect(d->m_currentParser, SIGNAL(finished()), this, SLOT(_q_handleParserFinished()));
-        disconnect(d->m_currentParser, SIGNAL(error(QPlaylistFileParser::ParserError, QString)),
-                   this, SLOT(_q_handleParserError(QPlaylistFileParser::ParserError, QString)));
+        disconnect(d->m_currentParser, SIGNAL(error(QPlaylistFileParser::ParserError,QString)),
+                   this, SLOT(_q_handleParserError(QPlaylistFileParser::ParserError,QString)));
         d->m_currentParser->deleteLater();
         d->m_currentParser = 0;
     }
