@@ -55,8 +55,6 @@
 
 #include <qtmultimediadefs.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 
@@ -72,7 +70,5 @@ public:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 
 #endif
