@@ -50,20 +50,15 @@
 
 QT_BEGIN_NAMESPACE
 
-
-namespace
+static void qRegisterRadioDataMetaTypes()
 {
-    class QRadioDataPrivateRegisterMetaTypes
-    {
-    public:
-        QRadioDataPrivateRegisterMetaTypes()
-        {
-            qRegisterMetaType<QRadioData::Error>();
-            qRegisterMetaType<QRadioData::ProgramType>();
-        }
-    } _registerMetaTypes;
+    qRegisterMetaType<QRadioData::Error>();
+    qRegisterMetaType<QRadioData::ProgramType>();
 }
 
+Q_CONSTRUCTOR_FUNCTION(qRegisterRadioDataMetaTypes)
+
+
 /*!
     \class QRadioData
     \brief The QRadioData class provides interfaces to the RDS functionality of the system radio.
