@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the Qt Toolkit.
@@ -43,20 +43,16 @@
 
 QT_BEGIN_NAMESPACE
 
-namespace
+static void qRegisterEncoderSettingsMetaTypes()
 {
-class QMediaEncoderSettingsPrivateRegisterMetaTypes
-{
-public:
-    QMediaEncoderSettingsPrivateRegisterMetaTypes()
-    {
-        qRegisterMetaType<QAudioEncoderSettings>();
-        qRegisterMetaType<QVideoEncoderSettings>();
-        qRegisterMetaType<QImageEncoderSettings>();
-    }
-} _registerMetaTypes;
+    qRegisterMetaType<QAudioEncoderSettings>();
+    qRegisterMetaType<QVideoEncoderSettings>();
+    qRegisterMetaType<QImageEncoderSettings>();
 }
 
+Q_CONSTRUCTOR_FUNCTION(qRegisterEncoderSettingsMetaTypes)
+
+
 class QAudioEncoderSettingsPrivate  : public QSharedData
 {
 public:
