@@ -48,19 +48,15 @@
 
 QT_BEGIN_NAMESPACE
 
-namespace
+static void qRegisterAbstractVideoBufferMetaTypes()
 {
-    class QAbstractVideoBufferPrivateRegisterMetaTypes
-    {
-    public:
-        QAbstractVideoBufferPrivateRegisterMetaTypes()
-        {
-            qRegisterMetaType<QAbstractVideoBuffer::HandleType>();
-            qRegisterMetaType<QAbstractVideoBuffer::MapMode>();
-        }
-    } _registerMetaTypes;
+    qRegisterMetaType<QAbstractVideoBuffer::HandleType>();
+    qRegisterMetaType<QAbstractVideoBuffer::MapMode>();
 }
 
+Q_CONSTRUCTOR_FUNCTION(qRegisterAbstractVideoBufferMetaTypes)
+
+
 /*!
     \class QAbstractVideoBuffer
     \brief The QAbstractVideoBuffer class is an abstraction for video data.
