@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the Qt Toolkit.
@@ -49,20 +49,15 @@
 
 QT_BEGIN_NAMESPACE
 
-namespace
+static void qRegisterVideoSurfaceFormatMetaTypes()
 {
-    class QVideoSurfaceFormatPrivateRegisterMetaTypes
-    {
-    public:
-        QVideoSurfaceFormatPrivateRegisterMetaTypes()
-        {
-            qRegisterMetaType<QVideoSurfaceFormat>();
-            qRegisterMetaType<QVideoSurfaceFormat::Direction>();
-            qRegisterMetaType<QVideoSurfaceFormat::YCbCrColorSpace>();
-        }
-    } _registerMetaTypes;
+    qRegisterMetaType<QVideoSurfaceFormat>();
+    qRegisterMetaType<QVideoSurfaceFormat::Direction>();
+    qRegisterMetaType<QVideoSurfaceFormat::YCbCrColorSpace>();
 }
 
+Q_CONSTRUCTOR_FUNCTION(qRegisterVideoSurfaceFormatMetaTypes)
+
 
 class QVideoSurfaceFormatPrivate : public QSharedData
 {
