@@ -247,6 +247,8 @@ public:
 
     bool isFormatSupported(const QVideoSurfaceFormat &format) const;
 
+    void stop();
+
     QAbstractVideoSurface::Error setCurrentFrame(const QVideoFrame &frame);
 
     QAbstractVideoSurface::Error paint(
@@ -351,6 +353,12 @@ bool QVideoSurfaceGLPainter::isFormatSupported(const QVideoSurfaceFormat &format
     return false;
 }
 
+
+void QVideoSurfaceGLPainter::stop()
+{
+    m_frame = QVideoFrame();
+}
+
 QAbstractVideoSurface::Error QVideoSurfaceGLPainter::setCurrentFrame(const QVideoFrame &frame)
 {
     m_frame = frame;
@@ -832,6 +840,8 @@ void QVideoSurfaceArbFpPainter::stop()
     m_textureCount = 0;
     m_programId = 0;
     m_handleType = QAbstractVideoBuffer::NoHandle;
+
+    QVideoSurfaceGLPainter::stop();
 }
 
 QAbstractVideoSurface::Error QVideoSurfaceArbFpPainter::paint(
@@ -841,7 +851,7 @@ QAbstractVideoSurface::Error QVideoSurfaceArbFpPainter::paint(
         painter->fillRect(target, Qt::black);
         return QAbstractVideoSurface::NoError;
     }
- 
+
     const QAbstractVideoBuffer::HandleType h = m_frame.handleType();
     if (h == QAbstractVideoBuffer::NoHandle || h == QAbstractVideoBuffer::GLTextureHandle) {
         bool stencilTestEnabled = glIsEnabled(GL_STENCIL_TEST);
@@ -1176,6 +1186,8 @@ void QVideoSurfaceGlslPainter::stop()
 
     m_textureCount = 0;
     m_handleType = QAbstractVideoBuffer::NoHandle;
+
+    QVideoSurfaceGLPainter::stop();
 }
 
 QAbstractVideoSurface::Error QVideoSurfaceGlslPainter::paint(
