@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the Qt Toolkit.
@@ -41,8 +41,8 @@
 
 #include "qandroidcamerasession.h"
 
-#include "jcamera.h"
-#include "jmultimediautils.h"
+#include "androidcamera.h"
+#include "androidmultimediautils.h"
 #include "qandroidvideooutput.h"
 #include "qandroidmediavideoprobecontrol.h"
 #include "qandroidmultimediautils.h"
@@ -57,15 +57,16 @@ QT_BEGIN_NAMESPACE
 class DataVideoBuffer : public QAbstractVideoBuffer
 {
 public:
-    DataVideoBuffer(const QByteArray &d)
+    DataVideoBuffer(const QByteArray &d, int bpl = -1)
         : QAbstractVideoBuffer(NoHandle)
         , data(d)
         , mode(NotMapped)
+        , bytesPerLine(bpl)
     { }
 
     MapMode mapMode() const { return mode; }
 
-    uchar *map(MapMode m, int *numBytes, int *bytesPerLine)
+    uchar *map(MapMode m, int *numBytes, int *bpl)
     {
         if (mode != NotMapped || m == NotMapped)
             return 0;
@@ -75,8 +76,8 @@ public:
         if (numBytes)
             *numBytes = data.size();
 
-        if (bytesPerLine)
-            *bytesPerLine = -1;
+        if (bpl)
+            *bpl = bytesPerLine;
 
         return reinterpret_cast<uchar *>(data.data());
     }
@@ -86,15 +87,16 @@ public:
 private:
     QByteArray data;
     MapMode mode;
+    int bytesPerLine;
 };
 
+Q_GLOBAL_STATIC(QList<AndroidCameraInfo>, g_availableCameras)
 
 QAndroidCameraSession::QAndroidCameraSession(QObject *parent)
     : QObject(parent)
     , m_selectedCamera(0)
     , m_camera(0)
     , m_nativeOrientation(0)
-    , m_previewOrientation(0)
     , m_videoOutput(0)
     , m_captureMode(QCamera::CaptureViewfinder)
     , m_state(QCamera::UnloadedState)
@@ -109,6 +111,10 @@ QAndroidCameraSession::QAndroidCameraSession(QObject *parent)
     , m_captureCanceled(false)
     , m_currentImageCaptureId(-1)
 {
+    m_mediaStorageLocation.addStorageLocation(
+                QMediaStorageLocation::Pictures,
+                AndroidMultimediaUtils::getDefaultMediaDirectory(AndroidMultimediaUtils::DCIM));
+
     if (qApp) {
         connect(qApp, SIGNAL(applicationStateChanged(Qt::ApplicationState)),
                 this, SLOT(onApplicationStateChanged(Qt::ApplicationState)));
@@ -162,10 +168,12 @@ void QAndroidCameraSession::setState(QCamera::State state)
             emit error(QCamera::CameraError, QStringLiteral("Failed to open camera"));
             return;
         }
-        if (state == QCamera::ActiveState)
-            startPreview();
-        else if (state == QCamera::LoadedState)
+        if (state == QCamera::ActiveState) {
+            if (!startPreview())
+                return;
+        } else if (state == QCamera::LoadedState) {
             stopPreview();
+        }
         break;
     }
 
@@ -173,6 +181,28 @@ void QAndroidCameraSession::setState(QCamera::State state)
      emit stateChanged(m_state);
 }
 
+void QAndroidCameraSession::updateAvailableCameras()
+{
+    g_availableCameras->clear();
+
+    const int numCameras = AndroidCamera::getNumberOfCameras();
+    for (int i = 0; i < numCameras; ++i) {
+        AndroidCameraInfo info;
+        AndroidCamera::getCameraInfo(i, &info);
+
+        if (!info.name.isNull())
+            g_availableCameras->append(info);
+    }
+}
+
+const QList<AndroidCameraInfo> &QAndroidCameraSession::availableCameras()
+{
+    if (g_availableCameras->isEmpty())
+        updateAvailableCameras();
+
+    return *g_availableCameras;
+}
+
 bool QAndroidCameraSession::open()
 {
     close();
@@ -180,7 +210,7 @@ bool QAndroidCameraSession::open()
     m_status = QCamera::LoadingStatus;
     emit statusChanged(m_status);
 
-    m_camera = JCamera::open(m_selectedCamera);
+    m_camera = AndroidCamera::open(m_selectedCamera);
 
     if (m_camera) {
         connect(m_camera, SIGNAL(pictureExposed()), this, SLOT(onCameraPictureExposed()));
@@ -194,16 +224,10 @@ bool QAndroidCameraSession::open()
 
         m_nativeOrientation = m_camera->getNativeOrientation();
 
-        // Preview orientation will always match the device natural orientation
-        if (m_camera->getFacing() == JCamera::CameraFacingFront)
-            m_previewOrientation = 360 - m_nativeOrientation;
-        else
-            m_previewOrientation = m_nativeOrientation;
-
         m_status = QCamera::LoadedStatus;
 
-        if (m_camera->getPreviewFormat() != JCamera::NV21)
-            m_camera->setPreviewFormat(JCamera::NV21);
+        if (m_camera->getPreviewFormat() != AndroidCamera::NV21)
+            m_camera->setPreviewFormat(AndroidCamera::NV21);
 
         m_camera->fetchEachFrame(m_videoProbes.count());
 
@@ -242,8 +266,10 @@ void QAndroidCameraSession::close()
 
 void QAndroidCameraSession::setVideoPreview(QObject *videoOutput)
 {
-    if (m_videoOutput)
+    if (m_videoOutput) {
         m_videoOutput->stop();
+        m_videoOutput->reset();
+    }
 
     if (videoOutput) {
         connect(videoOutput, SIGNAL(readyChanged(bool)), this, SLOT(onVideoOutputReady(bool)));
@@ -277,16 +303,8 @@ void QAndroidCameraSession::adjustViewfinderSize(const QSize &captureSize, bool
     }
 
     if (m_camera->previewSize() != viewfinderResolution) {
-        if (m_videoOutput) {
-            QSize size = viewfinderResolution;
-
-            // If the preview orientation is not the defaut one (0 or 180 degrees),
-            // we have to invert the output aspect ratio.
-            if (m_previewOrientation % 180)
-                size.transpose();
-
-            m_videoOutput->setVideoSize(size);
-        }
+        if (m_videoOutput)
+            m_videoOutput->setVideoSize(viewfinderResolution);
 
         // if preview is started, we have to stop it first before changing its size
         if (m_previewStarted && restartPreview)
@@ -300,25 +318,37 @@ void QAndroidCameraSession::adjustViewfinderSize(const QSize &captureSize, bool
     }
 }
 
-void QAndroidCameraSession::startPreview()
+bool QAndroidCameraSession::startPreview()
 {
-    if (!m_camera || m_previewStarted)
-        return;
+    if (!m_camera)
+        return false;
+
+    if (!m_videoOutput) {
+        Q_EMIT error(QCamera::InvalidRequestError, tr("Camera cannot be started without a viewfinder."));
+        return false;
+    }
+
+    if (m_previewStarted)
+        return true;
+
+    if (!m_videoOutput->isReady())
+        return true; // delay starting until the video output is ready
+
+    if (!m_camera->setPreviewTexture(m_videoOutput->surfaceTexture()))
+        return false;
 
     m_status = QCamera::StartingStatus;
     emit statusChanged(m_status);
 
     applyImageSettings();
     adjustViewfinderSize(m_imageSettings.resolution());
-    m_camera->setDisplayOrientation(m_previewOrientation);
 
-    if (m_videoOutput && m_videoOutput->isReady())
-        onVideoOutputReady(true);
-
-    JMultimediaUtils::enableOrientationListener(true);
+    AndroidMultimediaUtils::enableOrientationListener(true);
 
     m_camera->startPreview();
     m_previewStarted = true;
+
+    return true;
 }
 
 void QAndroidCameraSession::stopPreview()
@@ -329,12 +359,16 @@ void QAndroidCameraSession::stopPreview()
     m_status = QCamera::StoppingStatus;
     emit statusChanged(m_status);
 
-    JMultimediaUtils::enableOrientationListener(false);
+    AndroidMultimediaUtils::enableOrientationListener(false);
 
     m_camera->stopPreview();
     m_camera->setPreviewSize(QSize());
-    if (m_videoOutput)
+    m_camera->setPreviewTexture(0);
+
+    if (m_videoOutput) {
         m_videoOutput->stop();
+        m_videoOutput->reset();
+    }
     m_previewStarted = false;
 }
 
@@ -362,8 +396,8 @@ int QAndroidCameraSession::currentCameraRotation() const
 
     // subtract natural camera orientation and physical device orientation
     int rotation = 0;
-    int deviceOrientation = (JMultimediaUtils::getDeviceOrientation() + 45) / 90 * 90;
-    if (m_camera->getFacing() == JCamera::CameraFacingFront)
+    int deviceOrientation = (AndroidMultimediaUtils::getDeviceOrientation() + 45) / 90 * 90;
+    if (m_camera->getFacing() == AndroidCamera::CameraFacingFront)
         rotation = (m_nativeOrientation - deviceOrientation + 360) % 360;
     else // back-facing camera
         rotation = (m_nativeOrientation + deviceOrientation) % 360;
@@ -543,8 +577,11 @@ void QAndroidCameraSession::onCameraFrameFetched(const QByteArray &frame)
 {
     m_videoProbesMutex.lock();
     if (frame.size() && m_videoProbes.count()) {
-        QVideoFrame videoFrame(new DataVideoBuffer(frame),
-                               m_camera->previewSize(),
+        const QSize frameSize = m_camera->previewSize();
+        // Bytes per line should be only for the first plane. For NV21, the Y plane has 8 bits
+        // per sample, so bpl == width
+        QVideoFrame videoFrame(new DataVideoBuffer(frame, frameSize.width()),
+                               frameSize,
                                QVideoFrame::Format_NV21);
         foreach (QAndroidMediaVideoProbeControl *probe, m_videoProbes)
             probe->newFrameProbed(videoFrame);
@@ -600,7 +637,7 @@ void QAndroidCameraSession::processCapturedImage(int id,
 
     if (dest & QCameraImageCapture::CaptureToFile) {
         const QString actualFileName = m_mediaStorageLocation.generateFileName(fileName,
-                                                                               QAndroidMediaStorageLocation::Camera,
+                                                                               QMediaStorageLocation::Pictures,
                                                                                QLatin1String("IMG_"),
                                                                                QLatin1String("jpg"));
 
@@ -610,9 +647,9 @@ void QAndroidCameraSession::processCapturedImage(int id,
                 // if the picture is saved into the standard picture location, register it
                 // with the Android media scanner so it appears immediately in apps
                 // such as the gallery.
-                QString standardLoc = JMultimediaUtils::getDefaultMediaDirectory(JMultimediaUtils::DCIM);
+                QString standardLoc = AndroidMultimediaUtils::getDefaultMediaDirectory(AndroidMultimediaUtils::DCIM);
                 if (actualFileName.startsWith(standardLoc))
-                    JMultimediaUtils::registerMediaFile(actualFileName);
+                    AndroidMultimediaUtils::registerMediaFile(actualFileName);
 
                 emit imageSaved(id, actualFileName);
             } else {
@@ -649,7 +686,7 @@ QImage QAndroidCameraSession::prepareImageFromPreviewData(const QByteArray &data
     // Preview display of front-facing cameras is flipped horizontally, but the frame data
     // we get here is not. Flip it ourselves if the camera is front-facing to match what the user
     // sees on the viewfinder.
-    if (m_camera->getFacing() == JCamera::CameraFacingFront)
+    if (m_camera->getFacing() == AndroidCamera::CameraFacingFront)
         transform.scale(-1, 1);
 
     transform.rotate(rotation);
@@ -661,8 +698,8 @@ QImage QAndroidCameraSession::prepareImageFromPreviewData(const QByteArray &data
 
 void QAndroidCameraSession::onVideoOutputReady(bool ready)
 {
-    if (m_camera && m_videoOutput && ready)
-        m_camera->setPreviewTexture(m_videoOutput->surfaceTexture());
+    if (ready && m_state == QCamera::ActiveState)
+        startPreview();
 }
 
 void QAndroidCameraSession::onApplicationStateChanged(Qt::ApplicationState state)
