@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the Qt Toolkit.
@@ -41,11 +41,10 @@
 
 #include "qandroidcapturesession.h"
 
-#include "jcamera.h"
+#include "androidcamera.h"
 #include "qandroidcamerasession.h"
-#include "jmultimediautils.h"
+#include "androidmultimediautils.h"
 #include "qandroidmultimediautils.h"
-#include <QtCore/private/qjni_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -53,7 +52,7 @@ QAndroidCaptureSession::QAndroidCaptureSession(QAndroidCameraSession *cameraSess
     : QObject()
     , m_mediaRecorder(0)
     , m_cameraSession(cameraSession)
-    , m_audioSource(JMediaRecorder::DefaultAudioSource)
+    , m_audioSource(AndroidMediaRecorder::DefaultAudioSource)
     , m_duration(0)
     , m_state(QMediaRecorder::StoppedState)
     , m_status(QMediaRecorder::UnloadedStatus)
@@ -61,16 +60,28 @@ QAndroidCaptureSession::QAndroidCaptureSession(QAndroidCameraSession *cameraSess
     , m_containerFormatDirty(true)
     , m_videoSettingsDirty(true)
     , m_audioSettingsDirty(true)
-    , m_outputFormat(JMediaRecorder::DefaultOutputFormat)
-    , m_audioEncoder(JMediaRecorder::DefaultAudioEncoder)
-    , m_videoEncoder(JMediaRecorder::DefaultVideoEncoder)
+    , m_outputFormat(AndroidMediaRecorder::DefaultOutputFormat)
+    , m_audioEncoder(AndroidMediaRecorder::DefaultAudioEncoder)
+    , m_videoEncoder(AndroidMediaRecorder::DefaultVideoEncoder)
 {
+    m_mediaStorageLocation.addStorageLocation(
+                QMediaStorageLocation::Movies,
+                AndroidMultimediaUtils::getDefaultMediaDirectory(AndroidMultimediaUtils::DCIM));
+
+    m_mediaStorageLocation.addStorageLocation(
+                QMediaStorageLocation::Sounds,
+                AndroidMultimediaUtils::getDefaultMediaDirectory(AndroidMultimediaUtils::Sounds));
+
+    connect(this, SIGNAL(stateChanged(QMediaRecorder::State)), this, SLOT(updateStatus()));
+
     if (cameraSession) {
         connect(cameraSession, SIGNAL(opened()), this, SLOT(onCameraOpened()));
-        connect(cameraSession, SIGNAL(statusChanged(QCamera::Status)),
-                this, SLOT(onCameraStatusChanged(QCamera::Status)));
+        connect(cameraSession, SIGNAL(statusChanged(QCamera::Status)), this, SLOT(updateStatus()));
         connect(cameraSession, SIGNAL(captureModeChanged(QCamera::CaptureModes)),
-                this, SLOT(onCameraCaptureModeChanged(QCamera::CaptureModes)));
+                this, SLOT(updateStatus()));
+        connect(cameraSession, SIGNAL(readyForCaptureChanged(bool)), this, SLOT(updateStatus()));
+    } else {
+        updateStatus();
     }
 
     m_notifyTimer.setInterval(1000);
@@ -91,19 +102,19 @@ void QAndroidCaptureSession::setAudioInput(const QString &input)
     m_audioInput = input;
 
     if (m_audioInput == QLatin1String("default"))
-        m_audioSource = JMediaRecorder::DefaultAudioSource;
+        m_audioSource = AndroidMediaRecorder::DefaultAudioSource;
     else if (m_audioInput == QLatin1String("mic"))
-        m_audioSource = JMediaRecorder::Mic;
+        m_audioSource = AndroidMediaRecorder::Mic;
     else if (m_audioInput == QLatin1String("voice_uplink"))
-        m_audioSource = JMediaRecorder::VoiceUplink;
+        m_audioSource = AndroidMediaRecorder::VoiceUplink;
     else if (m_audioInput == QLatin1String("voice_downlink"))
-        m_audioSource = JMediaRecorder::VoiceDownlink;
+        m_audioSource = AndroidMediaRecorder::VoiceDownlink;
     else if (m_audioInput == QLatin1String("voice_call"))
-        m_audioSource = JMediaRecorder::VoiceCall;
+        m_audioSource = AndroidMediaRecorder::VoiceCall;
     else if (m_audioInput == QLatin1String("voice_recognition"))
-        m_audioSource = JMediaRecorder::VoiceRecognition;
+        m_audioSource = AndroidMediaRecorder::VoiceRecognition;
     else
-        m_audioSource = JMediaRecorder::DefaultAudioSource;
+        m_audioSource = AndroidMediaRecorder::DefaultAudioSource;
 
     emit audioInputChanged(m_audioInput);
 }
@@ -126,7 +137,6 @@ bool QAndroidCaptureSession::setOutputLocation(const QUrl &location)
 
     if (m_requestedOutputLocation.isValid()
             && (m_requestedOutputLocation.isLocalFile() || m_requestedOutputLocation.isRelative())) {
-        emit actualLocationChanged(m_requestedOutputLocation);
         return true;
     }
 
@@ -164,32 +174,26 @@ void QAndroidCaptureSession::setState(QMediaRecorder::State state)
 
 bool QAndroidCaptureSession::start()
 {
-    if (m_state == QMediaRecorder::RecordingState)
+    if (m_state == QMediaRecorder::RecordingState || m_status != QMediaRecorder::LoadedStatus)
         return false;
 
-    setStatus(QMediaRecorder::LoadingStatus);
+    setStatus(QMediaRecorder::StartingStatus);
 
     if (m_mediaRecorder) {
         m_mediaRecorder->release();
         delete m_mediaRecorder;
     }
-    m_mediaRecorder = new JMediaRecorder;
+    m_mediaRecorder = new AndroidMediaRecorder;
     connect(m_mediaRecorder, SIGNAL(error(int,int)), this, SLOT(onError(int,int)));
     connect(m_mediaRecorder, SIGNAL(info(int,int)), this, SLOT(onInfo(int,int)));
 
     // Set audio/video sources
     if (m_cameraSession) {
-        if (m_cameraSession->status() != QCamera::ActiveStatus) {
-            emit error(QMediaRecorder::ResourceError, QLatin1String("Camera must be active to record it."));
-            setStatus(QMediaRecorder::UnloadedStatus);
-            return false;
-        } else {
-            updateViewfinder();
-            m_cameraSession->camera()->unlock();
-            m_mediaRecorder->setCamera(m_cameraSession->camera());
-            m_mediaRecorder->setAudioSource(JMediaRecorder::Camcorder);
-            m_mediaRecorder->setVideoSource(JMediaRecorder::Camera);
-        }
+        updateViewfinder();
+        m_cameraSession->camera()->unlock();
+        m_mediaRecorder->setCamera(m_cameraSession->camera());
+        m_mediaRecorder->setAudioSource(AndroidMediaRecorder::Camcorder);
+        m_mediaRecorder->setVideoSource(AndroidMediaRecorder::Camera);
     } else {
         m_mediaRecorder->setAudioSource(m_audioSource);
     }
@@ -213,40 +217,31 @@ bool QAndroidCaptureSession::start()
         m_mediaRecorder->setOrientationHint(m_cameraSession->currentCameraRotation());
     }
 
-
     // Set output file
     QString filePath = m_mediaStorageLocation.generateFileName(
                 m_requestedOutputLocation.isLocalFile() ? m_requestedOutputLocation.toLocalFile()
                                                         : m_requestedOutputLocation.toString(),
-                m_cameraSession ? QAndroidMediaStorageLocation::Camera
-                                : QAndroidMediaStorageLocation::Audio,
+                m_cameraSession ? QMediaStorageLocation::Movies
+                                : QMediaStorageLocation::Sounds,
                 m_cameraSession ? QLatin1String("VID_")
                                 : QLatin1String("REC_"),
                 m_containerFormat);
 
-    m_actualOutputLocation = QUrl::fromLocalFile(filePath);
-    if (m_actualOutputLocation != m_requestedOutputLocation)
-        emit actualLocationChanged(m_actualOutputLocation);
-
+    m_usedOutputLocation = QUrl::fromLocalFile(filePath);
     m_mediaRecorder->setOutputFile(filePath);
 
     if (!m_mediaRecorder->prepare()) {
         emit error(QMediaRecorder::FormatError, QLatin1String("Unable to prepare the media recorder."));
-        setStatus(QMediaRecorder::UnloadedStatus);
+        restartViewfinder();
         return false;
     }
 
-    setStatus(QMediaRecorder::LoadedStatus);
-    setStatus(QMediaRecorder::StartingStatus);
-
     if (!m_mediaRecorder->start()) {
         emit error(QMediaRecorder::FormatError, QLatin1String("Unable to start the media recorder."));
-        setStatus(QMediaRecorder::UnloadedStatus);
+        restartViewfinder();
         return false;
     }
 
-    setStatus(QMediaRecorder::RecordingStatus);
-
     m_elapsedTime.start();
     m_notifyTimer.start();
     updateDuration();
@@ -270,29 +265,28 @@ void QAndroidCaptureSession::stop(bool error)
     updateDuration();
     m_elapsedTime.invalidate();
 
-    if (m_cameraSession) {
-        m_cameraSession->camera()->reconnect();
-        // Viewport needs to be restarted
-        m_cameraSession->camera()->startPreview();
-        m_cameraSession->setReadyForCapture(true);
-    }
-
     m_mediaRecorder->release();
     delete m_mediaRecorder;
     m_mediaRecorder = 0;
 
+    if (m_cameraSession && m_cameraSession->status() == QCamera::ActiveStatus) {
+        // Viewport needs to be restarted after recording
+        restartViewfinder();
+    }
+
     if (!error) {
         // if the media is saved into the standard media location, register it
         // with the Android media scanner so it appears immediately in apps
         // such as the gallery.
-        QString mediaPath = m_actualOutputLocation.toLocalFile();
-        QString standardLoc = m_cameraSession ? JMultimediaUtils::getDefaultMediaDirectory(JMultimediaUtils::DCIM)
-                                              : JMultimediaUtils::getDefaultMediaDirectory(JMultimediaUtils::Sounds);
+        QString mediaPath = m_usedOutputLocation.toLocalFile();
+        QString standardLoc = m_cameraSession ? AndroidMultimediaUtils::getDefaultMediaDirectory(AndroidMultimediaUtils::DCIM)
+                                              : AndroidMultimediaUtils::getDefaultMediaDirectory(AndroidMultimediaUtils::Sounds);
         if (mediaPath.startsWith(standardLoc))
-            JMultimediaUtils::registerMediaFile(mediaPath);
-    }
+            AndroidMultimediaUtils::registerMediaFile(mediaPath);
 
-    setStatus(QMediaRecorder::UnloadedStatus);
+        m_actualOutputLocation = m_usedOutputLocation;
+        emit actualLocationChanged(m_actualOutputLocation);
+    }
 }
 
 void QAndroidCaptureSession::setStatus(QMediaRecorder::Status status)
@@ -352,14 +346,14 @@ void QAndroidCaptureSession::applySettings()
             m_containerFormat = m_defaultSettings.outputFileExtension;
             m_outputFormat = m_defaultSettings.outputFormat;
         } else if (m_containerFormat == QLatin1String("3gp")) {
-            m_outputFormat = JMediaRecorder::THREE_GPP;
+            m_outputFormat = AndroidMediaRecorder::THREE_GPP;
         } else if (!m_cameraSession && m_containerFormat == QLatin1String("amr")) {
-            m_outputFormat = JMediaRecorder::AMR_NB_Format;
+            m_outputFormat = AndroidMediaRecorder::AMR_NB_Format;
         } else if (!m_cameraSession && m_containerFormat == QLatin1String("awb")) {
-            m_outputFormat = JMediaRecorder::AMR_WB_Format;
+            m_outputFormat = AndroidMediaRecorder::AMR_WB_Format;
         } else {
             m_containerFormat = QStringLiteral("mp4");
-            m_outputFormat = JMediaRecorder::MPEG_4;
+            m_outputFormat = AndroidMediaRecorder::MPEG_4;
         }
 
         m_containerFormatDirty = false;
@@ -377,11 +371,11 @@ void QAndroidCaptureSession::applySettings()
         if (m_audioSettings.codec().isEmpty())
             m_audioEncoder = m_defaultSettings.audioEncoder;
         else if (m_audioSettings.codec() == QLatin1String("aac"))
-            m_audioEncoder = JMediaRecorder::AAC;
+            m_audioEncoder = AndroidMediaRecorder::AAC;
         else if (m_audioSettings.codec() == QLatin1String("amr-nb"))
-            m_audioEncoder = JMediaRecorder::AMR_NB_Encoder;
+            m_audioEncoder = AndroidMediaRecorder::AMR_NB_Encoder;
         else if (m_audioSettings.codec() == QLatin1String("amr-wb"))
-            m_audioEncoder = JMediaRecorder::AMR_WB_Encoder;
+            m_audioEncoder = AndroidMediaRecorder::AMR_WB_Encoder;
         else
             m_audioEncoder = m_defaultSettings.audioEncoder;
 
@@ -389,7 +383,7 @@ void QAndroidCaptureSession::applySettings()
     }
 
     // video settings
-    if (m_cameraSession && m_videoSettingsDirty) {
+    if (m_cameraSession && m_cameraSession->camera() && m_videoSettingsDirty) {
         if (m_videoSettings.resolution().isEmpty()) {
             m_videoSettings.setResolution(m_defaultSettings.videoResolution);
             m_resolutionDirty = true;
@@ -415,11 +409,11 @@ void QAndroidCaptureSession::applySettings()
         if (m_videoSettings.codec().isEmpty())
             m_videoEncoder = m_defaultSettings.videoEncoder;
         else if (m_videoSettings.codec() == QLatin1String("h263"))
-            m_videoEncoder = JMediaRecorder::H263;
+            m_videoEncoder = AndroidMediaRecorder::H263;
         else if (m_videoSettings.codec() == QLatin1String("h264"))
-            m_videoEncoder = JMediaRecorder::H264;
+            m_videoEncoder = AndroidMediaRecorder::H264;
         else if (m_videoSettings.codec() == QLatin1String("mpeg4_sp"))
-            m_videoEncoder = JMediaRecorder::MPEG_4_SP;
+            m_videoEncoder = AndroidMediaRecorder::MPEG_4_SP;
         else
             m_videoEncoder = m_defaultSettings.videoEncoder;
 
@@ -437,6 +431,13 @@ void QAndroidCaptureSession::updateViewfinder()
     m_resolutionDirty = false;
 }
 
+void QAndroidCaptureSession::restartViewfinder()
+{
+    m_cameraSession->camera()->reconnect();
+    m_cameraSession->camera()->startPreview();
+    m_cameraSession->setReadyForCapture(true);
+}
+
 void QAndroidCaptureSession::updateDuration()
 {
     if (m_elapsedTime.isValid())
@@ -454,7 +455,7 @@ void QAndroidCaptureSession::onCameraOpened()
     for (int i = 0; i < 8; ++i) {
         CaptureProfile profile = getProfile(i);
         if (!profile.isNull) {
-            if (i == 1) // QUALITY_HIGH
+            if (i == AndroidCamcorderProfile::QUALITY_HIGH)
                 m_defaultSettings = profile;
 
             if (!m_supportedResolutions.contains(profile.videoResolution))
@@ -466,43 +467,38 @@ void QAndroidCaptureSession::onCameraOpened()
 
     qSort(m_supportedResolutions.begin(), m_supportedResolutions.end(), qt_sizeLessThan);
     qSort(m_supportedFramerates.begin(), m_supportedFramerates.end());
+
+    applySettings();
 }
 
 QAndroidCaptureSession::CaptureProfile QAndroidCaptureSession::getProfile(int id)
 {
     CaptureProfile profile;
-    bool hasProfile = QJNIObjectPrivate::callStaticMethod<jboolean>("android/media/CamcorderProfile",
-                                                             "hasProfile",
-                                                             "(II)Z",
-                                                             m_cameraSession->camera()->cameraId(),
-                                                             id);
+    const bool hasProfile = AndroidCamcorderProfile::hasProfile(m_cameraSession->camera()->cameraId(),
+                                                                AndroidCamcorderProfile::Quality(id));
 
     if (hasProfile) {
-        QJNIObjectPrivate obj = QJNIObjectPrivate::callStaticObjectMethod("android/media/CamcorderProfile",
-                                                                          "get",
-                                                                          "(II)Landroid/media/CamcorderProfile;",
-                                                                          m_cameraSession->camera()->cameraId(),
-                                                                          id);
+        AndroidCamcorderProfile camProfile = AndroidCamcorderProfile::get(m_cameraSession->camera()->cameraId(),
+                                                                          AndroidCamcorderProfile::Quality(id));
 
+        profile.outputFormat = AndroidMediaRecorder::OutputFormat(camProfile.getValue(AndroidCamcorderProfile::fileFormat));
+        profile.audioEncoder = AndroidMediaRecorder::AudioEncoder(camProfile.getValue(AndroidCamcorderProfile::audioCodec));
+        profile.audioBitRate = camProfile.getValue(AndroidCamcorderProfile::audioBitRate);
+        profile.audioChannels = camProfile.getValue(AndroidCamcorderProfile::audioChannels);
+        profile.audioSampleRate = camProfile.getValue(AndroidCamcorderProfile::audioSampleRate);
+        profile.videoEncoder = AndroidMediaRecorder::VideoEncoder(camProfile.getValue(AndroidCamcorderProfile::videoCodec));
+        profile.videoBitRate = camProfile.getValue(AndroidCamcorderProfile::videoBitRate);
+        profile.videoFrameRate = camProfile.getValue(AndroidCamcorderProfile::videoFrameRate);
+        profile.videoResolution = QSize(camProfile.getValue(AndroidCamcorderProfile::videoFrameWidth),
+                                        camProfile.getValue(AndroidCamcorderProfile::videoFrameHeight));
 
-        profile.outputFormat = JMediaRecorder::OutputFormat(obj.getField<jint>("fileFormat"));
-        profile.audioEncoder = JMediaRecorder::AudioEncoder(obj.getField<jint>("audioCodec"));
-        profile.audioBitRate = obj.getField<jint>("audioBitRate");
-        profile.audioChannels = obj.getField<jint>("audioChannels");
-        profile.audioSampleRate = obj.getField<jint>("audioSampleRate");
-        profile.videoEncoder = JMediaRecorder::VideoEncoder(obj.getField<jint>("videoCodec"));
-        profile.videoBitRate = obj.getField<jint>("videoBitRate");
-        profile.videoFrameRate = obj.getField<jint>("videoFrameRate");
-        profile.videoResolution = QSize(obj.getField<jint>("videoFrameWidth"),
-                                        obj.getField<jint>("videoFrameHeight"));
-
-        if (profile.outputFormat == JMediaRecorder::MPEG_4)
+        if (profile.outputFormat == AndroidMediaRecorder::MPEG_4)
             profile.outputFileExtension = QStringLiteral("mp4");
-        else if (profile.outputFormat == JMediaRecorder::THREE_GPP)
+        else if (profile.outputFormat == AndroidMediaRecorder::THREE_GPP)
             profile.outputFileExtension = QStringLiteral("3gp");
-        else if (profile.outputFormat == JMediaRecorder::AMR_NB_Format)
+        else if (profile.outputFormat == AndroidMediaRecorder::AMR_NB_Format)
             profile.outputFileExtension = QStringLiteral("amr");
-        else if (profile.outputFormat == JMediaRecorder::AMR_WB_Format)
+        else if (profile.outputFormat == AndroidMediaRecorder::AMR_WB_Format)
             profile.outputFileExtension = QStringLiteral("awb");
 
         profile.isNull = false;
@@ -511,16 +507,41 @@ QAndroidCaptureSession::CaptureProfile QAndroidCaptureSession::getProfile(int id
     return profile;
 }
 
-void QAndroidCaptureSession::onCameraStatusChanged(QCamera::Status status)
+void QAndroidCaptureSession::updateStatus()
 {
-    if (status == QCamera::StoppingStatus)
-        setState(QMediaRecorder::StoppedState);
-}
+    if (m_cameraSession) {
+        // Video recording
 
-void QAndroidCaptureSession::onCameraCaptureModeChanged(QCamera::CaptureModes mode)
-{
-    if (!mode.testFlag(QCamera::CaptureVideo))
-        setState(QMediaRecorder::StoppedState);
+        // stop recording when stopping the camera
+        if (m_cameraSession->status() == QCamera::StoppingStatus
+                || !m_cameraSession->captureMode().testFlag(QCamera::CaptureVideo)) {
+            setState(QMediaRecorder::StoppedState);
+            return;
+        }
+
+        if (m_state == QMediaRecorder::RecordingState) {
+            setStatus(QMediaRecorder::RecordingStatus);
+        } else if (m_cameraSession->status() == QCamera::UnavailableStatus) {
+            setStatus(QMediaRecorder::UnavailableStatus);
+        } else if (m_cameraSession->captureMode().testFlag(QCamera::CaptureVideo)
+                   && m_cameraSession->isReadyForCapture()) {
+            if (m_cameraSession->status() == QCamera::StartingStatus)
+                setStatus(QMediaRecorder::LoadingStatus);
+            else if (m_cameraSession->status() == QCamera::ActiveStatus)
+                setStatus(QMediaRecorder::LoadedStatus);
+            else
+                setStatus(QMediaRecorder::UnloadedStatus);
+        } else {
+            setStatus(QMediaRecorder::UnloadedStatus);
+        }
+
+    } else {
+        // Audio-only recording
+        if (m_state == QMediaRecorder::RecordingState)
+            setStatus(QMediaRecorder::RecordingStatus);
+        else
+            setStatus(QMediaRecorder::LoadedStatus);
+    }
 }
 
 void QAndroidCaptureSession::onError(int what, int extra)
