@@ -40,42 +40,49 @@
 ****************************************************************************/
 
 #include "qandroidmediaplayercontrol.h"
-#include "jmediaplayer.h"
+#include "androidmediaplayer.h"
 #include "qandroidvideooutput.h"
 
 QT_BEGIN_NAMESPACE
 
 QAndroidMediaPlayerControl::QAndroidMediaPlayerControl(QObject *parent)
     : QMediaPlayerControl(parent),
-      mMediaPlayer(new JMediaPlayer),
+      mMediaPlayer(new AndroidMediaPlayer),
       mCurrentState(QMediaPlayer::StoppedState),
       mCurrentMediaStatus(QMediaPlayer::NoMedia),
       mMediaStream(0),
       mVideoOutput(0),
       mSeekable(true),
       mBufferPercent(-1),
+      mBufferFilled(false),
       mAudioAvailable(false),
       mVideoAvailable(false),
       mBuffering(false),
-      mMediaPlayerReady(false),
+      mState(AndroidMediaPlayer::Uninitialized),
+      mPendingState(-1),
       mPendingPosition(-1),
-      mPendingSetMedia(false)
+      mPendingSetMedia(false),
+      mPendingVolume(-1),
+      mPendingMute(-1)
 {
-    connect(mMediaPlayer, SIGNAL(bufferingUpdate(qint32)),
-            this, SLOT(onBufferChanged(qint32)));
-    connect(mMediaPlayer, SIGNAL(info(qint32, qint32)),
-            this, SLOT(onInfo(qint32, qint32)));
-    connect(mMediaPlayer, SIGNAL(error(qint32, qint32)),
-            this, SLOT(onError(qint32, qint32)));
-    connect(mMediaPlayer, SIGNAL(mediaPlayerInfo(qint32, qint32)),
-            this, SLOT(onMediaPlayerInfo(qint32, qint32)));
-    connect(mMediaPlayer, SIGNAL(videoSizeChanged(qint32,qint32)),
-            this, SLOT(onVideoSizeChanged(qint32,qint32)));
+    connect(mMediaPlayer,SIGNAL(bufferingChanged(qint32)),
+            this,SLOT(onBufferingChanged(qint32)));
+    connect(mMediaPlayer,SIGNAL(info(qint32,qint32)),
+            this,SLOT(onInfo(qint32,qint32)));
+    connect(mMediaPlayer,SIGNAL(error(qint32,qint32)),
+            this,SLOT(onError(qint32,qint32)));
+    connect(mMediaPlayer,SIGNAL(stateChanged(qint32)),
+            this,SLOT(onStateChanged(qint32)));
+    connect(mMediaPlayer,SIGNAL(videoSizeChanged(qint32,qint32)),
+            this,SLOT(onVideoSizeChanged(qint32,qint32)));
+    connect(mMediaPlayer,SIGNAL(progressChanged(qint64)),
+            this,SIGNAL(positionChanged(qint64)));
+    connect(mMediaPlayer,SIGNAL(durationChanged(qint64)),
+            this,SIGNAL(durationChanged(qint64)));
 }
 
 QAndroidMediaPlayerControl::~QAndroidMediaPlayerControl()
 {
-    mMediaPlayer->stop();
     mMediaPlayer->release();
     delete mMediaPlayer;
 }
@@ -92,18 +99,33 @@ QMediaPlayer::MediaStatus QAndroidMediaPlayerControl::mediaStatus() const
 
 qint64 QAndroidMediaPlayerControl::duration() const
 {
-    return (mCurrentMediaStatus == QMediaPlayer::InvalidMedia
-            || mCurrentMediaStatus == QMediaPlayer::NoMedia
-            || !mMediaPlayerReady) ? 0
-                                   : mMediaPlayer->getDuration();
+    if ((mState & (AndroidMediaPlayer::Prepared
+                   | AndroidMediaPlayer::Started
+                   | AndroidMediaPlayer::Paused
+                   | AndroidMediaPlayer::Stopped
+                   | AndroidMediaPlayer::PlaybackCompleted)) == 0) {
+        return 0;
+    }
+
+    return mMediaPlayer->getDuration();
 }
 
 qint64 QAndroidMediaPlayerControl::position() const
 {
-    if (!mMediaPlayerReady)
-        return mPendingPosition < 0 ? 0 : mPendingPosition;
+    if (mCurrentMediaStatus == QMediaPlayer::EndOfMedia)
+        return duration();
 
-    return mMediaPlayer->getCurrentPosition();
+    if ((mState & (AndroidMediaPlayer::Idle
+                   | AndroidMediaPlayer::Initialized
+                   | AndroidMediaPlayer::Prepared
+                   | AndroidMediaPlayer::Started
+                   | AndroidMediaPlayer::Paused
+                   | AndroidMediaPlayer::Stopped
+                   | AndroidMediaPlayer::PlaybackCompleted)) == 0) {
+         return (mPendingPosition == -1) ? 0 : mPendingPosition;
+    }
+
+    return (mCurrentState == QMediaPlayer::StoppedState) ? 0 : mMediaPlayer->getCurrentPosition();
 }
 
 void QAndroidMediaPlayerControl::setPosition(qint64 position)
@@ -113,41 +135,94 @@ void QAndroidMediaPlayerControl::setPosition(qint64 position)
 
     const int seekPosition = (position > INT_MAX) ? INT_MAX : position;
 
-    if (!mMediaPlayerReady) {
-        mPendingPosition = seekPosition;
-        Q_EMIT positionChanged(seekPosition);
+    if ((mState & (AndroidMediaPlayer::Prepared
+                   | AndroidMediaPlayer::Started
+                   | AndroidMediaPlayer::Paused
+                   | AndroidMediaPlayer::PlaybackCompleted)) == 0) {
+        if (mPendingPosition != seekPosition) {
+            mPendingPosition = seekPosition;
+            Q_EMIT positionChanged(seekPosition);
+        }
         return;
     }
 
+    if (mCurrentMediaStatus == QMediaPlayer::EndOfMedia)
+        setMediaStatus(QMediaPlayer::LoadedMedia);
+
     mMediaPlayer->seekTo(seekPosition);
-    mPendingPosition = -1;
+
+    if (mPendingPosition != -1) {
+        mPendingPosition = -1;
+    }
+
+    Q_EMIT positionChanged(seekPosition);
 }
 
 int QAndroidMediaPlayerControl::volume() const
 {
-    return mMediaPlayer->volume();
+    return (mPendingVolume == -1) ? mMediaPlayer->volume() : mPendingVolume;
 }
 
 void QAndroidMediaPlayerControl::setVolume(int volume)
 {
+    if ((mState & (AndroidMediaPlayer::Idle
+                   | AndroidMediaPlayer::Initialized
+                   | AndroidMediaPlayer::Stopped
+                   | AndroidMediaPlayer::Prepared
+                   | AndroidMediaPlayer::Started
+                   | AndroidMediaPlayer::Paused
+                   | AndroidMediaPlayer::PlaybackCompleted)) == 0) {
+        if (mPendingVolume != volume) {
+            mPendingVolume = volume;
+            Q_EMIT volumeChanged(volume);
+        }
+        return;
+    }
+
     mMediaPlayer->setVolume(volume);
+
+    if (mPendingVolume != -1) {
+        mPendingVolume = -1;
+        return;
+    }
+
     Q_EMIT volumeChanged(volume);
 }
 
 bool QAndroidMediaPlayerControl::isMuted() const
 {
-    return mMediaPlayer->isMuted();
+    return (mPendingMute == -1) ? mMediaPlayer->isMuted() : (mPendingMute == 1);
 }
 
 void QAndroidMediaPlayerControl::setMuted(bool muted)
 {
+    if ((mState & (AndroidMediaPlayer::Idle
+                   | AndroidMediaPlayer::Initialized
+                   | AndroidMediaPlayer::Stopped
+                   | AndroidMediaPlayer::Prepared
+                   | AndroidMediaPlayer::Started
+                   | AndroidMediaPlayer::Paused
+                   | AndroidMediaPlayer::PlaybackCompleted)) == 0) {
+        if (mPendingMute != muted) {
+            mPendingMute = muted;
+            Q_EMIT mutedChanged(muted);
+        }
+        return;
+    }
+
     mMediaPlayer->setMuted(muted);
+
+    if (mPendingMute != -1) {
+        mPendingMute = -1;
+        return;
+    }
+
     Q_EMIT mutedChanged(muted);
 }
 
 int QAndroidMediaPlayerControl::bufferStatus() const
 {
-    return mBufferPercent;
+    return mBufferFilled ? 100 : 0;
 }
 
 bool QAndroidMediaPlayerControl::isAudioAvailable() const
@@ -208,10 +283,23 @@ const QIODevice *QAndroidMediaPlayerControl::mediaStream() const
 void QAndroidMediaPlayerControl::setMedia(const QMediaContent &mediaContent,
                                           QIODevice *stream)
 {
-    mMediaContent = mediaContent;
-    mMediaStream = stream;
+    const bool reloading = (mMediaContent == mediaContent);
 
-    if (mVideoOutput && !mMediaPlayer->display()) {
+    if (!reloading) {
+        mMediaContent = mediaContent;
+        mMediaStream = stream;
+    }
+
+    // Release the mediaplayer if it's not in in Idle or Uninitialized state
+    if ((mState & (AndroidMediaPlayer::Idle | AndroidMediaPlayer::Uninitialized)) == 0)
+        mMediaPlayer->release();
+
+    if (mediaContent.isNull()) {
+        setMediaStatus(QMediaPlayer::NoMedia);
+        return;
+    }
+
+    if (mVideoOutput && !mVideoOutput->isReady()) {
         // if a video output is set but the video texture is not ready, delay loading the media
         // since it can cause problems on some hardware
         mPendingSetMedia = true;
@@ -224,195 +312,189 @@ void QAndroidMediaPlayerControl::setMedia(const QMediaContent &mediaContent,
         const QString path = url.toString().mid(3);
         mTempFile.reset(QTemporaryFile::createNativeFile(path));
         if (!mTempFile.isNull())
-            mediaPath = QLatin1String("file://") + mTempFile->fileName();
+            mediaPath = QStringLiteral("file://") + mTempFile->fileName();
     } else {
         mediaPath = url.toString();
     }
 
-    if (!mediaPath.isEmpty())
-        mMediaPlayer->setDataSource(mediaPath);
-    else
-        setMediaStatus(QMediaPlayer::NoMedia);
+    if (mVideoSize.isValid() && mVideoOutput)
+        mVideoOutput->setVideoSize(mVideoSize);
 
-    Q_EMIT mediaChanged(mMediaContent);
+    if ((mMediaPlayer->display() == 0) && mVideoOutput)
+        mMediaPlayer->setDisplay(mVideoOutput->surfaceTexture());
+    mMediaPlayer->setDataSource(mediaPath);
+    mMediaPlayer->prepareAsync();
+
+    if (!reloading) {
+        Q_EMIT mediaChanged(mMediaContent);
+        Q_EMIT actualMediaLocationChanged(mediaPath);
+    }
 
     resetBufferingProgress();
-
-    // reset some properties
-    setAudioAvailable(false);
-    setVideoAvailable(false);
-    setSeekable(true);
 }
 
 void QAndroidMediaPlayerControl::setVideoOutput(QObject *videoOutput)
 {
-    if (mVideoOutput)
+    if (mVideoOutput) {
+        mMediaPlayer->setDisplay(0);
         mVideoOutput->stop();
+        mVideoOutput->reset();
+    }
 
     mVideoOutput = qobject_cast<QAndroidVideoOutput *>(videoOutput);
 
-    if (mVideoOutput && !mMediaPlayer->display()) {
-        if (mVideoOutput->isReady())
-            mMediaPlayer->setDisplay(mVideoOutput->surfaceHolder());
-        else
-            connect(videoOutput, SIGNAL(readyChanged(bool)), this, SLOT(onVideoOutputReady(bool)));
-    }
+    if (!mVideoOutput)
+        return;
+
+    if (mVideoOutput->isReady())
+        mMediaPlayer->setDisplay(mVideoOutput->surfaceTexture());
+
+    connect(videoOutput, SIGNAL(readyChanged(bool)), this, SLOT(onVideoOutputReady(bool)));
 }
 
 void QAndroidMediaPlayerControl::play()
 {
-    if (!mMediaPlayerReady) {
+    // We need to prepare the mediaplayer again.
+    if ((mState & AndroidMediaPlayer::Stopped) && !mMediaContent.isNull()) {
+        setMedia(mMediaContent, mMediaStream);
+    }
+
+    setState(QMediaPlayer::PlayingState);
+
+    if ((mState & (AndroidMediaPlayer::Prepared
+                   | AndroidMediaPlayer::Started
+                   | AndroidMediaPlayer::Paused
+                   | AndroidMediaPlayer::PlaybackCompleted)) == 0) {
         mPendingState = QMediaPlayer::PlayingState;
-        if (mCurrentState == QMediaPlayer::StoppedState
-            && !mMediaContent.isNull()
-            && mCurrentMediaStatus != QMediaPlayer::LoadingMedia
-            && !mPendingSetMedia) {
-            setMedia(mMediaContent, 0);
-        }
         return;
     }
 
     mMediaPlayer->play();
-    setState(QMediaPlayer::PlayingState);
 }
 
 void QAndroidMediaPlayerControl::pause()
 {
-    if (!mMediaPlayerReady) {
+    setState(QMediaPlayer::PausedState);
+
+    if ((mState & (AndroidMediaPlayer::Started
+                   | AndroidMediaPlayer::Paused
+                   | AndroidMediaPlayer::PlaybackCompleted)) == 0) {
         mPendingState = QMediaPlayer::PausedState;
         return;
     }
 
     mMediaPlayer->pause();
-    setState(QMediaPlayer::PausedState);
 }
 
 void QAndroidMediaPlayerControl::stop()
 {
-    mMediaPlayer->stop();
     setState(QMediaPlayer::StoppedState);
+
+    if ((mState & (AndroidMediaPlayer::Prepared
+                   | AndroidMediaPlayer::Started
+                   | AndroidMediaPlayer::Stopped
+                   | AndroidMediaPlayer::Paused
+                   | AndroidMediaPlayer::PlaybackCompleted)) == 0) {
+        if ((mState & (AndroidMediaPlayer::Idle | AndroidMediaPlayer::Uninitialized | AndroidMediaPlayer::Error)) == 0)
+            mPendingState = QMediaPlayer::StoppedState;
+        return;
+    }
+
+    mMediaPlayer->stop();
 }
 
 void QAndroidMediaPlayerControl::onInfo(qint32 what, qint32 extra)
 {
     Q_UNUSED(extra);
     switch (what) {
-    case JMediaPlayer::MEDIA_INFO_UNKNOWN:
+    case AndroidMediaPlayer::MEDIA_INFO_UNKNOWN:
         break;
-    case JMediaPlayer::MEDIA_INFO_VIDEO_TRACK_LAGGING:
+    case AndroidMediaPlayer::MEDIA_INFO_VIDEO_TRACK_LAGGING:
         // IGNORE
         break;
-    case JMediaPlayer::MEDIA_INFO_VIDEO_RENDERING_START:
+    case AndroidMediaPlayer::MEDIA_INFO_VIDEO_RENDERING_START:
         break;
-    case JMediaPlayer::MEDIA_INFO_BUFFERING_START:
+    case AndroidMediaPlayer::MEDIA_INFO_BUFFERING_START:
         mPendingState = mCurrentState;
         setState(QMediaPlayer::PausedState);
         setMediaStatus(QMediaPlayer::StalledMedia);
         break;
-    case JMediaPlayer::MEDIA_INFO_BUFFERING_END:
-        setMediaStatus(mBufferPercent == 100 ? QMediaPlayer::BufferedMedia : QMediaPlayer::BufferingMedia);
-        flushPendingStates();
+    case AndroidMediaPlayer::MEDIA_INFO_BUFFERING_END:
+        if (mCurrentState != QMediaPlayer::StoppedState)
+            flushPendingStates();
         break;
-    case JMediaPlayer::MEDIA_INFO_BAD_INTERLEAVING:
+    case AndroidMediaPlayer::MEDIA_INFO_BAD_INTERLEAVING:
         break;
-    case JMediaPlayer::MEDIA_INFO_NOT_SEEKABLE:
+    case AndroidMediaPlayer::MEDIA_INFO_NOT_SEEKABLE:
         setSeekable(false);
         break;
-    case JMediaPlayer::MEDIA_INFO_METADATA_UPDATE:
+    case AndroidMediaPlayer::MEDIA_INFO_METADATA_UPDATE:
         Q_EMIT metaDataUpdated();
         break;
     }
 }
 
-void QAndroidMediaPlayerControl::onMediaPlayerInfo(qint32 what, qint32 extra)
-{
-    switch (what) {
-    case JMediaPlayer::MEDIA_PLAYER_INVALID_STATE:
-        setError(what, QStringLiteral("Error: Invalid state"));
-        break;
-    case JMediaPlayer::MEDIA_PLAYER_PREPARING:
-        setMediaStatus(QMediaPlayer::LoadingMedia);
-        setState(QMediaPlayer::StoppedState);
-        break;
-    case JMediaPlayer::MEDIA_PLAYER_READY:
-        setMediaStatus(QMediaPlayer::LoadedMedia);
-        if (mBuffering) {
-            setMediaStatus(mBufferPercent == 100 ? QMediaPlayer::BufferedMedia
-                                                 : QMediaPlayer::BufferingMedia);
-        } else {
-            onBufferChanged(100);
-        }
-        setAudioAvailable(true);
-        mMediaPlayerReady = true;
-        flushPendingStates();
-        break;
-    case JMediaPlayer::MEDIA_PLAYER_DURATION:
-        Q_EMIT durationChanged(extra);
-        break;
-    case JMediaPlayer::MEDIA_PLAYER_PROGRESS:
-        Q_EMIT positionChanged(extra);
-        break;
-    case JMediaPlayer::MEDIA_PLAYER_FINISHED:
-        stop();
-        setMediaStatus(QMediaPlayer::EndOfMedia);
-        break;
-    }
-}
-
 void QAndroidMediaPlayerControl::onError(qint32 what, qint32 extra)
 {
     QString errorString;
     QMediaPlayer::Error error = QMediaPlayer::ResourceError;
 
     switch (what) {
-    case JMediaPlayer::MEDIA_ERROR_UNKNOWN:
+    case AndroidMediaPlayer::MEDIA_ERROR_UNKNOWN:
         errorString = QLatin1String("Error:");
         break;
-    case JMediaPlayer::MEDIA_ERROR_SERVER_DIED:
+    case AndroidMediaPlayer::MEDIA_ERROR_SERVER_DIED:
         errorString = QLatin1String("Error: Server died");
         error = QMediaPlayer::ServiceMissingError;
         break;
+    case AndroidMediaPlayer::MEDIA_ERROR_INVALID_STATE:
+        errorString = QLatin1String("Error: Invalid state");
+        error = QMediaPlayer::ServiceMissingError;
+        break;
     }
 
     switch (extra) {
-    case JMediaPlayer::MEDIA_ERROR_IO: // Network OR file error
+    case AndroidMediaPlayer::MEDIA_ERROR_IO: // Network OR file error
         errorString += QLatin1String(" (I/O operation failed)");
         error = QMediaPlayer::NetworkError;
         setMediaStatus(QMediaPlayer::InvalidMedia);
         break;
-    case JMediaPlayer::MEDIA_ERROR_MALFORMED:
+    case AndroidMediaPlayer::MEDIA_ERROR_MALFORMED:
         errorString += QLatin1String(" (Malformed bitstream)");
         error = QMediaPlayer::FormatError;
         setMediaStatus(QMediaPlayer::InvalidMedia);
         break;
-    case JMediaPlayer::MEDIA_ERROR_UNSUPPORTED:
+    case AndroidMediaPlayer::MEDIA_ERROR_UNSUPPORTED:
         errorString += QLatin1String(" (Unsupported media)");
         error = QMediaPlayer::FormatError;
         setMediaStatus(QMediaPlayer::InvalidMedia);
         break;
-    case JMediaPlayer::MEDIA_ERROR_TIMED_OUT:
+    case AndroidMediaPlayer::MEDIA_ERROR_TIMED_OUT:
         errorString += QLatin1String(" (Timed out)");
         break;
-    case JMediaPlayer::MEDIA_ERROR_NOT_VALID_FOR_PROGRESSIVE_PLAYBACK:
+    case AndroidMediaPlayer::MEDIA_ERROR_NOT_VALID_FOR_PROGRESSIVE_PLAYBACK:
         errorString += QLatin1String(" (Unable to start progressive playback')");
         error = QMediaPlayer::FormatError;
         setMediaStatus(QMediaPlayer::InvalidMedia);
         break;
+    case AndroidMediaPlayer::MEDIA_ERROR_BAD_THINGS_ARE_GOING_TO_HAPPEN:
+        errorString += QLatin1String(" (Unknown error/Insufficient resources)");
+        error = QMediaPlayer::ServiceMissingError;
+        break;
     }
 
-    setError(error, errorString);
+    Q_EMIT QMediaPlayerControl::error(error, errorString);
 }
 
-void QAndroidMediaPlayerControl::onBufferChanged(qint32 percent)
+void QAndroidMediaPlayerControl::onBufferingChanged(qint32 percent)
 {
     mBuffering = percent != 100;
     mBufferPercent = percent;
-    Q_EMIT bufferStatusChanged(mBufferPercent);
 
     updateAvailablePlaybackRanges();
 
-    if (mBufferPercent == 100)
-        setMediaStatus(QMediaPlayer::BufferedMedia);
+    if (mCurrentState != QMediaPlayer::StoppedState)
+        setMediaStatus(mBuffering ? QMediaPlayer::BufferingMedia : QMediaPlayer::BufferedMedia);
 }
 
 void QAndroidMediaPlayerControl::onVideoSizeChanged(qint32 width, qint32 height)
@@ -429,12 +511,91 @@ void QAndroidMediaPlayerControl::onVideoSizeChanged(qint32 width, qint32 height)
         mVideoOutput->setVideoSize(mVideoSize);
 }
 
+void QAndroidMediaPlayerControl::onStateChanged(qint32 state)
+{
+    // If reloading, don't report state changes unless the new state is Prepared or Error.
+    if ((mState & AndroidMediaPlayer::Stopped)
+        && (state & (AndroidMediaPlayer::Prepared | AndroidMediaPlayer::Error | AndroidMediaPlayer::Uninitialized)) == 0) {
+        return;
+    }
+
+    mState = state;
+    switch (mState) {
+    case AndroidMediaPlayer::Idle:
+        break;
+    case AndroidMediaPlayer::Initialized:
+        break;
+    case AndroidMediaPlayer::Preparing:
+        setMediaStatus(QMediaPlayer::LoadingMedia);
+        break;
+    case AndroidMediaPlayer::Prepared:
+        setMediaStatus(QMediaPlayer::LoadedMedia);
+        if (mBuffering) {
+            setMediaStatus(mBufferPercent == 100 ? QMediaPlayer::BufferedMedia
+                                                 : QMediaPlayer::BufferingMedia);
+        } else {
+            onBufferingChanged(100);
+        }
+        setAudioAvailable(true);
+        flushPendingStates();
+        break;
+    case AndroidMediaPlayer::Started:
+        setState(QMediaPlayer::PlayingState);
+        if (mBuffering) {
+            setMediaStatus(mBufferPercent == 100 ? QMediaPlayer::BufferedMedia
+                                                 : QMediaPlayer::BufferingMedia);
+        } else {
+            setMediaStatus(QMediaPlayer::BufferedMedia);
+        }
+        break;
+    case AndroidMediaPlayer::Paused:
+        setState(QMediaPlayer::PausedState);
+        break;
+    case AndroidMediaPlayer::Error:
+        setState(QMediaPlayer::StoppedState);
+        setMediaStatus(QMediaPlayer::UnknownMediaStatus);
+        mMediaPlayer->release();
+        break;
+    case AndroidMediaPlayer::Stopped:
+        setState(QMediaPlayer::StoppedState);
+        setMediaStatus(QMediaPlayer::LoadedMedia);
+        setPosition(0);
+        break;
+    case AndroidMediaPlayer::PlaybackCompleted:
+        setState(QMediaPlayer::StoppedState);
+        setPosition(0);
+        setMediaStatus(QMediaPlayer::EndOfMedia);
+        break;
+    case AndroidMediaPlayer::Uninitialized:
+        // reset some properties
+        resetBufferingProgress();
+        mPendingPosition = -1;
+        mPendingSetMedia = false;
+        mPendingState = -1;
+
+        setAudioAvailable(false);
+        setVideoAvailable(false);
+        setSeekable(true);
+        break;
+    default:
+        break;
+    }
+
+    if ((mState & (AndroidMediaPlayer::Stopped | AndroidMediaPlayer::Uninitialized)) != 0) {
+        mMediaPlayer->setDisplay(0);
+        if (mVideoOutput) {
+            mVideoOutput->stop();
+            mVideoOutput->reset();
+        }
+    }
+}
+
 void QAndroidMediaPlayerControl::onVideoOutputReady(bool ready)
 {
-    if (!mMediaPlayer->display() && mVideoOutput && ready) {
-        mMediaPlayer->setDisplay(mVideoOutput->surfaceHolder());
-        flushPendingStates();
-    }
+    if ((mMediaPlayer->display() == 0) && mVideoOutput && ready)
+        mMediaPlayer->setDisplay(mVideoOutput->surfaceTexture());
+
+    flushPendingStates();
 }
 
 void QAndroidMediaPlayerControl::setState(QMediaPlayer::State state)
@@ -442,14 +603,8 @@ void QAndroidMediaPlayerControl::setState(QMediaPlayer::State state)
     if (mCurrentState == state)
         return;
 
-    if (state == QMediaPlayer::StoppedState) {
-        if (mVideoOutput)
-            mVideoOutput->stop();
-        resetBufferingProgress();
-        mMediaPlayerReady = false;
-        mPendingPosition = -1;
-        Q_EMIT positionChanged(0);
-    }
+    if (mCurrentState == QMediaPlayer::StoppedState && state == QMediaPlayer::PausedState)
+        return;
 
     mCurrentState = state;
     Q_EMIT stateChanged(mCurrentState);
@@ -463,15 +618,13 @@ void QAndroidMediaPlayerControl::setMediaStatus(QMediaPlayer::MediaStatus status
     if (status == QMediaPlayer::NoMedia || status == QMediaPlayer::InvalidMedia)
         Q_EMIT durationChanged(0);
 
+    if (status == QMediaPlayer::EndOfMedia)
+        Q_EMIT durationChanged(duration());
+
     mCurrentMediaStatus = status;
     Q_EMIT mediaStatusChanged(mCurrentMediaStatus);
-}
 
-void QAndroidMediaPlayerControl::setError(int error,
-                                          const QString &errorString)
-{
-    setState(QMediaPlayer::StoppedState);
-    Q_EMIT QMediaPlayerControl::error(error, errorString);
+    updateBufferStatus();
 }
 
 void QAndroidMediaPlayerControl::setSeekable(bool seekable)
@@ -509,21 +662,28 @@ void QAndroidMediaPlayerControl::resetBufferingProgress()
     mBuffering = false;
     mBufferPercent = 0;
     mAvailablePlaybackRange = QMediaTimeRange();
-    Q_EMIT bufferStatusChanged(mBufferPercent);
 }
 
 void QAndroidMediaPlayerControl::flushPendingStates()
 {
     if (mPendingSetMedia) {
-        setMedia(mMediaContent, 0);
         mPendingSetMedia = false;
+        setMedia(mMediaContent, 0);
         return;
     }
 
-    switch (mPendingState) {
+    const int newState = mPendingState;
+    mPendingState = -1;
+
+    if (mPendingPosition != -1)
+        setPosition(mPendingPosition);
+    if (mPendingVolume != -1)
+        setVolume(mPendingVolume);
+    if (mPendingMute != -1)
+        setMuted((mPendingMute == 1));
+
+    switch (newState) {
     case QMediaPlayer::PlayingState:
-        if (mPendingPosition > -1)
-            setPosition(mPendingPosition);
         play();
         break;
     case QMediaPlayer::PausedState:
@@ -532,6 +692,19 @@ void QAndroidMediaPlayerControl::flushPendingStates()
     case QMediaPlayer::StoppedState:
         stop();
         break;
+    default:
+        break;
+    }
+}
+
+void QAndroidMediaPlayerControl::updateBufferStatus()
+{
+    bool bufferFilled = (mCurrentMediaStatus == QMediaPlayer::BufferedMedia
+                         || mCurrentMediaStatus == QMediaPlayer::BufferingMedia);
+
+    if (mBufferFilled != bufferFilled) {
+        mBufferFilled = bufferFilled;
+        Q_EMIT bufferStatusChanged(bufferStatus());
     }
 }
 
