@@ -250,7 +250,7 @@ QString AudioCaptureSession::generateFileName(const QDir &dir,
 {
     int lastClip = 0;
     foreach(QString fileName, dir.entryList(QStringList() << QString("clip_*.%1").arg(ext))) {
-        int imgNumber = fileName.mid(5, fileName.size()-6-ext.length()).toInt();
+        int imgNumber = fileName.midRef(5, fileName.size()-6-ext.length()).toInt();
         lastClip = qMax(lastClip, imgNumber);
     }
 
