@@ -79,15 +79,32 @@ extern const CLSID CLSID_VideoInputDeviceCategory;
 
 QT_USE_NAMESPACE
 
+static int g_refCount = 0;
+void addRefCount()
+{
+    if (++g_refCount == 1)
+        CoInitialize(NULL);
+}
+
+void releaseRefCount()
+{
+    if (--g_refCount == 0)
+        CoUninitialize();
+}
+
 QMediaService* DSServicePlugin::create(QString const& key)
 {
 #ifdef QMEDIA_DIRECTSHOW_CAMERA
-    if (key == QLatin1String(Q_MEDIASERVICE_CAMERA))
+    if (key == QLatin1String(Q_MEDIASERVICE_CAMERA)) {
+        addRefCount();
         return new DSCameraService;
+    }
 #endif
 #ifdef QMEDIA_DIRECTSHOW_PLAYER
-    if (key == QLatin1String(Q_MEDIASERVICE_MEDIAPLAYER))
+    if (key == QLatin1String(Q_MEDIASERVICE_MEDIAPLAYER)) {
+        addRefCount();
         return new DirectShowPlayerService;
+    }
 #endif
 
     return 0;
@@ -96,6 +113,7 @@ QMediaService* DSServicePlugin::create(QString const& key)
 void DSServicePlugin::release(QMediaService *service)
 {
     delete service;
+    releaseRefCount();
 }
 
 QMediaServiceProviderHint::Features DSServicePlugin::supportedFeatures(
@@ -107,6 +125,20 @@ QMediaServiceProviderHint::Features DSServicePlugin::supportedFeatures(
         return QMediaServiceProviderHint::Features();
 }
 
+QByteArray DSServicePlugin::defaultDevice(const QByteArray &service) const
+{
+#ifdef QMEDIA_DIRECTSHOW_CAMERA
+    if (service == Q_MEDIASERVICE_CAMERA) {
+        if (m_cameraDevices.isEmpty())
+            updateDevices();
+
+        return m_defaultCameraDevice;
+    }
+#endif
+
+    return QByteArray();
+}
+
 QList<QByteArray> DSServicePlugin::devices(const QByteArray &service) const
 {
 #ifdef QMEDIA_DIRECTSHOW_CAMERA
@@ -140,11 +172,18 @@ QString DSServicePlugin::deviceDescription(const QByteArray &service, const QByt
 
 void DSServicePlugin::updateDevices() const
 {
+    addRefCount();
+
+    m_defaultCameraDevice.clear();
     DSVideoDeviceControl::enumerateDevices(&m_cameraDevices, &m_cameraDescriptions);
 
     if (m_cameraDevices.isEmpty()) {
         qWarning() << "No camera devices found";
+    } else {
+        m_defaultCameraDevice = m_cameraDevices.first();
     }
+
+    releaseRefCount();
 }
 #endif
 
