@@ -111,7 +111,6 @@ DirectShowPlayerService::DirectShowPlayerService(QObject *parent)
     , m_seekable(false)
     , m_atEnd(false)
 {
-    CoInitialize(NULL);
     m_playerControl = new DirectShowPlayerControl(this);
     m_metaDataControl = new DirectShowMetaDataControl(this);
     m_audioEndpointControl = new DirectShowAudioEndpointControl(this);
@@ -153,7 +152,6 @@ DirectShowPlayerService::~DirectShowPlayerService()
 #endif
 
     ::CloseHandle(m_taskHandle);
-    CoUninitialize();
 }
 
 QMediaControl *DirectShowPlayerService::requestControl(const char *name)
@@ -563,7 +561,7 @@ void DirectShowPlayerService::releaseGraph()
             }
             m_graph->Abort();
         }
-        
+
         m_pendingTasks = ReleaseGraph;
 
         ::SetEvent(m_taskHandle);
@@ -803,7 +801,7 @@ void DirectShowPlayerService::doStop(QMutexLocker *locker)
         m_pendingTasks |= Seek;
 
         m_executedTasks &= ~(Play | Pause);
-        
+
         QCoreApplication::postEvent(this, new QEvent(QEvent::Type(StatusChange)));
     }
 
@@ -852,7 +850,7 @@ void DirectShowPlayerService::doSetRate(QMutexLocker *locker)
 
         seeking->Release();
     } else if (m_rate != 1.0) {
-        m_rate = 1.0;   
+        m_rate = 1.0;
     }
     QCoreApplication::postEvent(this, new QEvent(QEvent::Type(RateChange)));
 }
@@ -980,7 +978,7 @@ void DirectShowPlayerService::setAudioOutput(IBaseFilter *filter)
                 m_loop->wait(&m_mutex);
             }
             m_audioOutput->Release();
-        } 
+        }
 
         m_audioOutput = filter;
 
