@@ -48,6 +48,7 @@
 #include <private/gstvideoconnector_p.h>
 #include <private/qgstutils_p.h>
 #include <private/playlistfileparser_p.h>
+#include <private/qgstutils_p.h>
 
 #include <gst/gstvalue.h>
 #include <gst/base/gstbasesrc.h>
@@ -66,6 +67,20 @@
 
 QT_BEGIN_NAMESPACE
 
+static bool usePlaybinVolume()
+{
+    static enum { Yes, No, Unknown } status = Unknown;
+    if (status == Unknown) {
+        QByteArray v = qgetenv("QT_GSTREAMER_USE_PLAYBIN_VOLUME");
+        bool value = !v.isEmpty() && v != "0" && v != "false";
+        if (value)
+            status = Yes;
+        else
+            status = No;
+    }
+    return status == Yes;
+}
+
 typedef enum {
     GST_PLAY_FLAG_VIDEO         = 0x00000001,
     GST_PLAY_FLAG_AUDIO         = 0x00000002,
@@ -103,6 +118,7 @@ QGstreamerPlayerSession::QGstreamerPlayerSession(QObject *parent)
      m_pendingVideoSink(0),
      m_nullVideoSink(0),
      m_audioSink(0),
+     m_volumeElement(0),
      m_bus(0),
      m_videoOutput(0),
      m_renderer(0),
@@ -150,24 +166,47 @@ QGstreamerPlayerSession::QGstreamerPlayerSession(QObject *parent)
 #endif
         g_object_set(G_OBJECT(m_playbin), "flags", flags, NULL);
 
-        m_audioSink = gst_element_factory_make("autoaudiosink", "audiosink");
-        if (m_audioSink) {
+        GstElement *audioSink = gst_element_factory_make("autoaudiosink", "audiosink");
+        if (audioSink) {
+            if (usePlaybinVolume()) {
+                m_audioSink = audioSink;
+                m_volumeElement = m_playbin;
+            } else {
+                m_volumeElement = gst_element_factory_make("volume", "volumeelement");
+                if (m_volumeElement) {
+                    m_audioSink = gst_bin_new("audio-output-bin");
+
+                    gst_bin_add_many(GST_BIN(m_audioSink), m_volumeElement, audioSink, NULL);
+                    gst_element_link(m_volumeElement, audioSink);
+
+                    GstPad *pad = gst_element_get_static_pad(m_volumeElement, "sink");
+                    gst_element_add_pad(GST_ELEMENT(m_audioSink), gst_ghost_pad_new("sink", pad));
+                    gst_object_unref(GST_OBJECT(pad));
+                } else {
+                    m_audioSink = audioSink;
+                    m_volumeElement = m_playbin;
+                }
+            }
+
             g_object_set(G_OBJECT(m_playbin), "audio-sink", m_audioSink, NULL);
             addAudioBufferProbe();
         }
     }
 
-    m_videoOutputBin = gst_bin_new("video-output-bin");
-    gst_object_ref(GST_OBJECT(m_videoOutputBin));
-
-    m_videoIdentity = GST_ELEMENT(g_object_new(gst_video_connector_get_type(), 0));
+    m_videoIdentity = GST_ELEMENT(g_object_new(gst_video_connector_get_type(), 0)); // floating ref
     g_signal_connect(G_OBJECT(m_videoIdentity), "connection-failed", G_CALLBACK(insertColorSpaceElement), (gpointer)this);
+
     m_colorSpace = gst_element_factory_make("ffmpegcolorspace", "ffmpegcolorspace-vo");
-    gst_object_ref(GST_OBJECT(m_colorSpace));
+    // might not get a parent, take ownership to avoid leak
+    qt_gst_object_ref_sink(GST_OBJECT(m_colorSpace));
 
     m_nullVideoSink = gst_element_factory_make("fakesink", NULL);
     g_object_set(G_OBJECT(m_nullVideoSink), "sync", true, NULL);
     gst_object_ref(GST_OBJECT(m_nullVideoSink));
+
+    m_videoOutputBin = gst_bin_new("video-output-bin");
+    // might not get a parent, take ownership to avoid leak
+    qt_gst_object_ref_sink(GST_OBJECT(m_videoOutputBin));
     gst_bin_add_many(GST_BIN(m_videoOutputBin), m_videoIdentity, m_nullVideoSink, NULL);
     gst_element_link(m_videoIdentity, m_nullVideoSink);
 
@@ -189,12 +228,12 @@ QGstreamerPlayerSession::QGstreamerPlayerSession(QObject *parent)
         g_signal_connect(G_OBJECT(m_playbin), "notify::source", G_CALLBACK(playbinNotifySource), this);
         g_signal_connect(G_OBJECT(m_playbin), "element-added",  G_CALLBACK(handleElementAdded), this);
 
-        // Init volume and mute state
-        g_object_set(G_OBJECT(m_playbin), "volume", 1.0, NULL);
-        g_object_set(G_OBJECT(m_playbin), "mute", FALSE, NULL);
-
-        g_signal_connect(G_OBJECT(m_playbin), "notify::volume", G_CALLBACK(handleVolumeChange), this);
-        g_signal_connect(G_OBJECT(m_playbin), "notify::mute", G_CALLBACK(handleMutedChange), this);
+        if (usePlaybinVolume()) {
+            updateVolume();
+            updateMuted();
+            g_signal_connect(G_OBJECT(m_playbin), "notify::volume", G_CALLBACK(handleVolumeChange), this);
+            g_signal_connect(G_OBJECT(m_playbin), "notify::mute", G_CALLBACK(handleMutedChange), this);
+        }
 
         g_signal_connect(G_OBJECT(m_playbin), "video-changed", G_CALLBACK(handleStreamsChange), this);
         g_signal_connect(G_OBJECT(m_playbin), "audio-changed", G_CALLBACK(handleStreamsChange), this);
@@ -238,6 +277,8 @@ void QGstreamerPlayerSession::configureAppSrcElement(GObject* object, GObject *o
 
     if (!self->appsrc()->setup(appsrc))
         qWarning()<<"Could not setup appsrc element";
+
+    g_object_unref(G_OBJECT(appsrc));
 }
 #endif
 
@@ -327,7 +368,7 @@ void QGstreamerPlayerSession::setPlaybackRate(qreal rate)
 #endif
     if (!qFuzzyCompare(m_playbackRate, rate)) {
         m_playbackRate = rate;
-        if (m_playbin) {
+        if (m_playbin && m_seekable) {
             gst_element_seek(m_playbin, rate, GST_FORMAT_TIME,
                              GstSeekFlags(GST_SEEK_FLAG_FLUSH),
                              GST_SEEK_TYPE_NONE,0,
@@ -491,7 +532,7 @@ void QGstreamerPlayerSession::setVideoRenderer(QObject *videoOutput)
         }
     }
 
-    QGstreamerVideoRendererInterface* renderer = qobject_cast<QGstreamerVideoRendererInterface*>(videoOutput);   
+    QGstreamerVideoRendererInterface* renderer = qobject_cast<QGstreamerVideoRendererInterface*>(videoOutput);
 
     m_renderer = renderer;
 
@@ -646,7 +687,7 @@ void QGstreamerPlayerSession::finishVideoOutputChange()
         m_pendingVideoSink = 0;
         gst_object_unref(GST_OBJECT(srcPad));
         return;
-    }  
+    }
 
     if (m_usingColorspaceElement) {
         gst_element_set_state(m_colorSpace, GST_STATE_NULL);
@@ -877,7 +918,7 @@ bool QGstreamerPlayerSession::seek(qint64 ms)
     qDebug() << Q_FUNC_INFO << ms;
 #endif
     //seek locks when the video output sink is changing and pad is blocked
-    if (m_playbin && !m_pendingVideoSink && m_state != QMediaPlayer::StoppedState) {
+    if (m_playbin && !m_pendingVideoSink && m_state != QMediaPlayer::StoppedState && m_seekable) {
         ms = qMax(ms,qint64(0));
         gint64  position = ms * 1000000;
         bool isSeeking = gst_element_seek(m_playbin,
@@ -906,10 +947,8 @@ void QGstreamerPlayerSession::setVolume(int volume)
     if (m_volume != volume) {
         m_volume = volume;
 
-        if (m_playbin) {
-            //playbin2 allows to set volume and muted independently,
-            g_object_set(G_OBJECT(m_playbin), "volume", m_volume/100.0, NULL);
-        }
+        if (m_volumeElement)
+            g_object_set(G_OBJECT(m_volumeElement), "volume", m_volume / 100.0, NULL);
 
         emit volumeChanged(m_volume);
     }
@@ -923,7 +962,9 @@ void QGstreamerPlayerSession::setMuted(bool muted)
     if (m_muted != muted) {
         m_muted = muted;
 
-        g_object_set(G_OBJECT(m_playbin), "mute", m_muted ? TRUE : FALSE, NULL);
+        if (m_volumeElement)
+            g_object_set(G_OBJECT(m_volumeElement), "mute", m_muted ? TRUE : FALSE, NULL);
+
         emit mutedStateChanged(m_muted);
     }
 }
@@ -946,13 +987,15 @@ bool QGstreamerPlayerSession::processBusMessage(const QGstreamerMessage &message
     if (gm) {
         //tag message comes from elements inside playbin, not from playbin itself
         if (GST_MESSAGE_TYPE(gm) == GST_MESSAGE_TAG) {
-            //qDebug() << "tag message";
             GstTagList *tag_list;
             gst_message_parse_tag(gm, &tag_list);
-            m_tags.unite(QGstUtils::gstTagListToMap(tag_list));
+
+            QMap<QByteArray, QVariant> newTags = QGstUtils::gstTagListToMap(tag_list);
+            QMap<QByteArray, QVariant>::const_iterator it = newTags.constBegin();
+            for ( ; it != newTags.constEnd(); ++it)
+                m_tags.insert(it.key(), it.value()); // overwrite existing tags
 
             gst_tag_list_free(tag_list);
-            //qDebug() << m_tags;
 
             emit tagsChanged();
         } else if (GST_MESSAGE_TYPE(gm) == GST_MESSAGE_DURATION) {
@@ -1418,13 +1461,6 @@ void QGstreamerPlayerSession::playbinNotifySource(GObject *o, GParamSpec *p, gpo
     qDebug() << "Playbin source added:" << G_OBJECT_CLASS_NAME(G_OBJECT_GET_CLASS(source));
 #endif
 
-    // Turn off icecast metadata request, will be re-set if in QNetworkRequest
-    // (souphttpsrc docs say is false by default, but header appears in request
-    // @version 0.10.21)
-    if (g_object_class_find_property(G_OBJECT_GET_CLASS(source), "iradio-mode") != 0)
-        g_object_set(G_OBJECT(source), "iradio-mode", FALSE, NULL);
-
-
     // Set Headers
     const QByteArray userAgentString("User-Agent");
 
@@ -1499,6 +1535,7 @@ void QGstreamerPlayerSession::playbinNotifySource(GObject *o, GParamSpec *p, gpo
         qDebug() << "Current source is a non-live source";
 #endif
 
+    g_object_set(G_OBJECT(self->m_videoSink), "sync", !self->m_isLiveSource, NULL);
 
     gst_object_unref(source);
 }
