@@ -93,7 +93,7 @@ MFPlayerSession::MFPlayerSession(MFPlayerService *playerService)
     , m_audioSampleGrabberNode(0)
     , m_videoProbeMFT(0)
 {
-    QObject::connect(this, SIGNAL(sessionEvent(IMFMediaEvent *)), this, SLOT(handleSessionEvent(IMFMediaEvent *)));
+    QObject::connect(this, SIGNAL(sessionEvent(IMFMediaEvent*)), this, SLOT(handleSessionEvent(IMFMediaEvent*)));
 
     m_pendingState = NoPending;
     ZeroMemory(&m_state, sizeof(m_state));
@@ -1157,8 +1157,8 @@ void MFPlayerSession::setPositionInternal(qint64 position, Command requestCmd)
 
 qreal MFPlayerSession::playbackRate() const
 {
-    if (m_pendingState != NoPending)
-        return m_request.rate;
+    if (m_scrubbing)
+        return m_restoreRate;
     return m_state.rate;
 }
 
@@ -1166,6 +1166,7 @@ void MFPlayerSession::setPlaybackRate(qreal rate)
 {
     if (m_scrubbing) {
         m_restoreRate = rate;
+        emit playbackRateChanged(rate);
         return;
     }
     setPlaybackRateInternal(rate);
@@ -1194,6 +1195,8 @@ void MFPlayerSession::setPlaybackRateInternal(qreal rate)
         isThin = TRUE;
         if (FAILED(m_rateSupport->IsRateSupported(isThin, rate, NULL))) {
             qWarning() << "unable to set playbackrate = " << rate;
+            m_pendingRate = m_request.rate = m_state.rate;
+            return;
         }
     }
     if (m_pendingState != NoPending) {
@@ -1219,6 +1222,7 @@ void MFPlayerSession::commitRateChange(qreal rate, BOOL isThin)
     MFTIME  hnsSystemTime = 0;
     MFTIME  hnsClockTime = 0;
     Command cmdNow = m_state.command;
+    bool resetPosition = false;
     // Allowed rate transitions:
     // Positive <-> negative:   Stopped
     // Negative <-> zero:       Stopped
@@ -1229,7 +1233,12 @@ void MFPlayerSession::commitRateChange(qreal rate, BOOL isThin)
             m_presentationClock->GetCorrelatedTime(0, &hnsClockTime, &hnsSystemTime);
             Q_ASSERT(hnsSystemTime != 0);
 
-            m_request.setCommand(rate < 0 || m_state.rate < 0 ? CmdSeekResume : CmdStart);
+            if (rate < 0 || m_state.rate < 0)
+                m_request.setCommand(CmdSeekResume);
+            else if (isThin || m_state.isThin)
+                m_request.setCommand(CmdStartAndSeek);
+            else
+                m_request.setCommand(CmdStart);
 
             // We need to stop only when dealing with negative rates
             if (rate >= 0 && m_state.rate >= 0)
@@ -1247,7 +1256,9 @@ void MFPlayerSession::commitRateChange(qreal rate, BOOL isThin)
                 // session cannot transition back from stopped to paused.
                 // Therefore, this rate transition is not supported while paused.
                 qWarning() << "Unable to change rate from positive to negative or vice versa in paused state";
-                return;
+                rate = m_state.rate;
+                isThin = m_state.isThin;
+                goto done;
             }
 
             // This happens when resuming playback after scrubbing in pause mode.
@@ -1279,17 +1290,42 @@ void MFPlayerSession::commitRateChange(qreal rate, BOOL isThin)
 
         // Resume to the current position (stop() will reset the position to 0)
         m_request.start = hnsClockTime / 10000;
+    } else if (!isThin && m_state.isThin) {
+        if (cmdNow == CmdStart) {
+            // When thinning, only key frames are read and presented. Going back
+            // to normal playback requires to reset the current position to force
+            // the pipeline to decode the actual frame at the current position
+            // (which might be earlier than the last decoded key frame)
+            resetPosition = true;
+        } else if (cmdNow == CmdPause) {
+            // If paused, dont reset the position until we resume, otherwise
+            // a new frame will be rendered
+            m_presentationClock->GetCorrelatedTime(0, &hnsClockTime, &hnsSystemTime);
+            m_request.setCommand(CmdSeekResume);
+            m_request.start = hnsClockTime / 10000;
+        }
+
     }
 
     // Set the rate.
     if (FAILED(m_rateControl->SetRate(isThin, rate))) {
         qWarning() << "failed to set playbackrate = " << rate;
-        return;
+        rate = m_state.rate;
+        isThin = m_state.isThin;
+        goto done;
     }
 
+    if (resetPosition) {
+        m_presentationClock->GetCorrelatedTime(0, &hnsClockTime, &hnsSystemTime);
+        setPosition(hnsClockTime / 10000);
+    }
+
+done:
     // Adjust our current rate and requested rate.
     m_pendingRate = m_request.rate = m_state.rate = rate;
-
+    if (rate != 0)
+        m_state.isThin = isThin;
+    emit playbackRateChanged(rate);
 }
 
 void MFPlayerSession::scrub(bool enableScrub)
@@ -1325,8 +1361,10 @@ void MFPlayerSession::setVolume(int volume)
     if (m_volume == volume)
         return;
     m_volume = volume;
-    if (m_volumeControl)
-        m_volumeControl->SetMasterVolume(m_volume * 0.01f);
+
+    if (!m_muted)
+        setVolumeInternal(volume);
+
     emit volumeChanged(m_volume);
 }
 
@@ -1340,24 +1378,42 @@ void MFPlayerSession::setMuted(bool muted)
     if (m_muted == muted)
         return;
     m_muted = muted;
-    if (m_volumeControl)
-        m_volumeControl->SetMute(BOOL(m_muted));
+
+    setVolumeInternal(muted ? 0 : m_volume);
+
     emit mutedChanged(m_muted);
 }
 
+void MFPlayerSession::setVolumeInternal(int volume)
+{
+    if (m_volumeControl) {
+        quint32 channelCount = 0;
+        if (!SUCCEEDED(m_volumeControl->GetChannelCount(&channelCount))
+                || channelCount == 0)
+            return;
+
+        float scaled = volume * 0.01f;
+        for (quint32 i = 0; i < channelCount; ++i)
+            m_volumeControl->SetChannelVolume(i, scaled);
+    }
+}
+
 int MFPlayerSession::bufferStatus()
 {
     if (!m_netsourceStatistics)
         return 0;
     PROPVARIANT var;
+    PropVariantInit(&var);
     PROPERTYKEY key;
     key.fmtid = MFNETSOURCE_STATISTICS;
     key.pid = MFNETSOURCE_BUFFERPROGRESS_ID;
     int progress = -1;
-    if (SUCCEEDED(m_netsourceStatistics->GetValue(key, &var))) {
+    // GetValue returns S_FALSE if the property is not available, which has
+    // a value > 0. We therefore can't use the SUCCEEDED macro here.
+    if (m_netsourceStatistics->GetValue(key, &var) == S_OK) {
         progress = var.lVal;
+        PropVariantClear(&var);
     }
-    PropVariantClear(&var);
 
 #ifdef DEBUG_MEDIAFOUNDATION
     qDebug() << "bufferStatus: progress = " << progress;
@@ -1368,22 +1424,30 @@ int MFPlayerSession::bufferStatus()
 
 QMediaTimeRange MFPlayerSession::availablePlaybackRanges()
 {
-    if (!m_netsourceStatistics)
-        return QMediaTimeRange();
+    // defaults to the whole media
+    qint64 start = 0;
+    qint64 end = qint64(m_duration / 10000);
 
-    qint64 start = 0, end = 0;
-    PROPVARIANT var;
-    PROPERTYKEY key;
-    key.fmtid = MFNETSOURCE_STATISTICS;
-    key.pid = MFNETSOURCE_SEEKRANGESTART_ID;
-    if (SUCCEEDED(m_netsourceStatistics->GetValue(key, &var))) {
-        start = qint64(var.uhVal.QuadPart / 10000);
-        key.pid = MFNETSOURCE_SEEKRANGEEND_ID;
-        if (SUCCEEDED(m_netsourceStatistics->GetValue(key, &var))) {
-            end = qint64(var.uhVal.QuadPart / 10000);
+    if (m_netsourceStatistics) {
+        PROPVARIANT var;
+        PropVariantInit(&var);
+        PROPERTYKEY key;
+        key.fmtid = MFNETSOURCE_STATISTICS;
+        key.pid = MFNETSOURCE_SEEKRANGESTART_ID;
+        // GetValue returns S_FALSE if the property is not available, which has
+        // a value > 0. We therefore can't use the SUCCEEDED macro here.
+        if (m_netsourceStatistics->GetValue(key, &var) == S_OK) {
+            start = qint64(var.uhVal.QuadPart / 10000);
+            PropVariantClear(&var);
+            PropVariantInit(&var);
+            key.pid = MFNETSOURCE_SEEKRANGEEND_ID;
+            if (m_netsourceStatistics->GetValue(key, &var) == S_OK) {
+                end = qint64(var.uhVal.QuadPart / 10000);
+                PropVariantClear(&var);
+            }
         }
     }
-    PropVariantClear(&var);
+
     return QMediaTimeRange(start, end);
 }
 
@@ -1570,10 +1634,8 @@ void MFPlayerSession::handleSessionEvent(IMFMediaEvent *sessionEvent)
                 }
             }
 
-            if (SUCCEEDED(MFGetService(m_session, MR_POLICY_VOLUME_SERVICE, IID_PPV_ARGS(&m_volumeControl)))) {
-                m_volumeControl->SetMasterVolume(m_volume * 0.01f);
-                m_volumeControl->SetMute(m_muted);
-            }
+            if (SUCCEEDED(MFGetService(m_session, MR_STREAM_VOLUME_SERVICE, IID_PPV_ARGS(&m_volumeControl))))
+                setVolumeInternal(m_muted ? 0 : m_volume);
 
             DWORD dwCharacteristics = 0;
             m_sourceResolver->mediaSource()->GetCharacteristics(&dwCharacteristics);
@@ -1619,25 +1681,6 @@ void MFPlayerSession::handleSessionEvent(IMFMediaEvent *sessionEvent)
         break;
     case MEEndOfPresentationSegment:
         break;
-    case MEAudioSessionVolumeChanged:
-        if (m_volumeControl) {
-            float currentVolume = 1;
-            if (SUCCEEDED(m_volumeControl->GetMasterVolume(&currentVolume))) {
-                int scaledVolume = currentVolume * 100;
-                if (scaledVolume != m_volume) {
-                    m_volume = scaledVolume;
-                    emit volumeChanged(scaledVolume);
-                }
-            }
-            BOOL currentMuted = FALSE;
-            if (SUCCEEDED(m_volumeControl->GetMute(&currentMuted))) {
-                if (currentMuted != BOOL(m_muted)) {
-                    m_muted = bool(currentMuted);
-                    emit mutedChanged(m_muted);
-                }
-            }
-        }
-        break;
     case MESessionTopologyStatus: {
             UINT32 status;
             if (SUCCEEDED(sessionEvent->GetUINT32(MF_EVENT_TOPOLOGY_STATUS, &status))) {
@@ -1709,6 +1752,11 @@ void MFPlayerSession::updatePendingCommands(Command command)
         case CmdSeek:
         case CmdSeekResume:
             setPositionInternal(m_request.start, m_request.command);
+            break;
+        case CmdStartAndSeek:
+            start();
+            setPositionInternal(m_request.start, m_request.command);
+            break;
         }
         m_request.setCommand(CmdNone);
     }
