@@ -257,6 +257,7 @@ namespace
             , m_bufferStartTime(-1)
             , m_bufferDuration(-1)
             , m_presentationClock(0)
+            , m_sampleRequested(false)
             , m_currentMediaType(0)
             , m_prerolling(false)
             , m_prerollTargetTime(0)
@@ -571,6 +572,18 @@ namespace
                         QVideoSurfaceFormat format(QSize(width, height), m_pixelFormats[index]);
                         m_surfaceFormat = format;
 
+                        MFVideoArea viewport;
+                        if (SUCCEEDED(pMediaType->GetBlob(MF_MT_GEOMETRIC_APERTURE,
+                                                          reinterpret_cast<UINT8*>(&viewport),
+                                                          sizeof(MFVideoArea),
+                                                          NULL))) {
+
+                            m_surfaceFormat.setViewport(QRect(viewport.OffsetX.value,
+                                                              viewport.OffsetY.value,
+                                                              viewport.Area.cx,
+                                                              viewport.Area.cy));
+                        }
+
                         if (FAILED(pMediaType->GetUINT32(MF_MT_DEFAULT_STRIDE, (UINT32*)&m_bytesPerLine))) {
                             m_bytesPerLine = getBytesPerLine(format);
                         }
@@ -654,11 +667,6 @@ namespace
                 m_presentationClock = NULL;
             }
 
-            if (m_scheduledBuffer) {
-                m_scheduledBuffer->Release();
-                m_scheduledBuffer = NULL;
-            }
-
             clearMediaTypes();
             clearSampleQueue();
             clearBufferCache();
@@ -676,6 +684,7 @@ namespace
             QMutexLocker locker(&m_mutex);
             HRESULT hr = validateOperation(OpPreroll);
             if (SUCCEEDED(hr)) {
+                m_state = State_Prerolling;
                 m_prerollTargetTime = hnsUpcomingStartTime;
                 hr = queueAsyncOperation(OpPreroll);
             }
@@ -771,9 +780,12 @@ namespace
             qDebug() << "MediaStream::setRate" << rate;
 #endif
             QMutexLocker locker(&m_mutex);
-            m_rate = rate;
-            queueEvent(MEStreamSinkRateChanged, GUID_NULL, S_OK, NULL);
-            return S_OK;
+            HRESULT hr = validateOperation(OpSetRate);
+            if (SUCCEEDED(hr)) {
+                m_rate = rate;
+                hr = queueAsyncOperation(OpSetRate);
+            }
+            return hr;
         }
 
         void supportedFormatsChanged()
@@ -854,6 +866,16 @@ namespace
                 schedulePresentation(true);
         }
 
+        void clearScheduledFrame()
+        {
+            QMutexLocker locker(&m_mutex);
+            if (m_scheduledBuffer) {
+                m_scheduledBuffer->Release();
+                m_scheduledBuffer = NULL;
+                schedulePresentation(true);
+            }
+        }
+
         enum
         {
             StartSurface = QEvent::User,
@@ -871,7 +893,7 @@ namespace
             {
             }
 
-            int targetTime()
+            MFTIME targetTime()
             {
                 return m_time;
             }
@@ -923,6 +945,7 @@ namespace
         {
             State_TypeNotSet = 0,    // No media type is set
             State_Ready,             // Media type is set, Start has never been called.
+            State_Prerolling,
             State_Started,
             State_Paused,
             State_Stopped,
@@ -1111,6 +1134,9 @@ namespace
                             break;
                         }
                     }
+
+                    if (m_state == State_Started)
+                        schedulePresentation(true);
                 case OpRestart:
                     endPreroll(S_FALSE);
                     if (SUCCEEDED(hr)) {
@@ -1129,10 +1155,7 @@ namespace
                 case OpStop:
                     // Drop samples from queue.
                     hr = processSamplesFromQueue(DropSamples);
-                    if (m_scheduledBuffer) {
-                        m_scheduledBuffer->Release();
-                        m_scheduledBuffer = NULL;
-                    }
+                    clearBufferCache();
                     // Send the event even if the previous call failed.
                     hr = queueEvent(MEStreamSinkStopped, GUID_NULL, hr, NULL);
                     if (m_surface->isActive()) {
@@ -1148,7 +1171,7 @@ namespace
                     hr = queueEvent(MEStreamSinkPaused, GUID_NULL, hr, NULL);
                     break;
                 case OpSetRate:
-                    //TODO:
+                    hr = queueEvent(MEStreamSinkRateChanged, GUID_NULL, S_OK, NULL);
                     break;
                 case OpProcessSample:
                 case OpPlaceMarker:
@@ -1317,13 +1340,15 @@ namespace
 
         HRESULT processSampleData(IMFSample *pSample)
         {
+            m_sampleRequested = false;
+
             LONGLONG time, duration = -1;
             HRESULT hr = pSample->GetSampleTime(&time);
             if (SUCCEEDED(hr))
                pSample->GetSampleDuration(&duration);
 
             if (m_prerolling) {
-                if (SUCCEEDED(hr) && time >= m_prerollTargetTime) {
+                if (SUCCEEDED(hr) && ((time - m_prerollTargetTime) * m_rate) >= 0) {
                     IMFMediaBuffer *pBuffer = NULL;
                     hr = pSample->ConvertToContiguousBuffer(&pBuffer);
                     if (SUCCEEDED(hr)) {
@@ -1340,7 +1365,7 @@ namespace
             } else {
                 bool requestSample = true;
                 // If the time stamp is too early, just discard this sample.
-                if (SUCCEEDED(hr) && time >= m_startTime) {
+                if (SUCCEEDED(hr) && ((time - m_startTime) * m_rate) >= 0) {
                     IMFMediaBuffer *pBuffer = NULL;
                     hr = pSample->ConvertToContiguousBuffer(&pBuffer);
                     if (SUCCEEDED(hr)) {
@@ -1373,11 +1398,16 @@ namespace
             foreach (SampleBuffer sb, m_bufferCache)
                 sb.m_buffer->Release();
             m_bufferCache.clear();
+
+            if (m_scheduledBuffer) {
+                m_scheduledBuffer->Release();
+                m_scheduledBuffer = NULL;
+            }
         }
 
         void schedulePresentation(bool requestSample)
         {
-            if (m_state == State_Paused)
+            if (m_state == State_Paused || m_state == State_Prerolling)
                 return;
             if (!m_scheduledBuffer) {
                 //get time from presentation time
@@ -1388,11 +1418,12 @@ namespace
                         timeOK = false;
                 }
                 while (!m_bufferCache.isEmpty()) {
-                    SampleBuffer sb = m_bufferCache.first();
-                    m_bufferCache.pop_front();
-                    if (timeOK && currentTime > sb.m_time) {
+                    SampleBuffer sb = m_bufferCache.takeFirst();
+                    if (timeOK && ((sb.m_time - currentTime) * m_rate) < 0) {
                         sb.m_buffer->Release();
+#ifdef DEBUG_MEDIAFOUNDATION
                         qDebug() << "currentPresentTime =" << float(currentTime / 10000) * 0.001f << " and sampleTime is" << float(sb.m_time / 10000) * 0.001f;
+#endif
                         continue;
                     }
                     m_scheduledBuffer = sb.m_buffer;
@@ -1404,13 +1435,16 @@ namespace
                     break;
                 }
             }
-            if (requestSample && m_bufferCache.size() < BUFFER_CACHE_SIZE)
+            if (requestSample && !m_sampleRequested && m_bufferCache.size() < BUFFER_CACHE_SIZE) {
+                m_sampleRequested = true;
                 queueEvent(MEStreamSinkRequestSample, GUID_NULL, S_OK, NULL);
+            }
         }
         IMFMediaBuffer *m_scheduledBuffer;
         MFTIME m_bufferStartTime;
         MFTIME m_bufferDuration;
         IMFPresentationClock *m_presentationClock;
+        bool m_sampleRequested;
         float m_rate;
     };
 
@@ -1422,6 +1456,8 @@ namespace
 
         /* Ready */                 TRUE,   TRUE,   TRUE,       FALSE,    TRUE,     TRUE,     TRUE,      FALSE,    TRUE,     TRUE,
 
+        /* Prerolling */            TRUE,   TRUE,   FALSE,    FALSE,     TRUE,    TRUE,      TRUE,      TRUE,      TRUE,     TRUE,
+
         /* Start */                 FALSE,  TRUE,   TRUE,      FALSE,    TRUE,     TRUE,     TRUE,       TRUE,     TRUE,     TRUE,
 
         /* Pause */                 FALSE,  TRUE,   TRUE,      TRUE,     TRUE,     TRUE,     TRUE,       TRUE,     TRUE,     TRUE,
@@ -1437,7 +1473,11 @@ namespace
         // 2. While paused, the sink accepts samples but does not process them.
     };
 
-    class MediaSink : public IMFFinalizableMediaSink, public IMFClockStateSink, public IMFMediaSinkPreroll
+    class MediaSink : public IMFFinalizableMediaSink,
+                      public IMFClockStateSink,
+                      public IMFMediaSinkPreroll,
+                      public IMFGetService,
+                      public IMFRateSupport
     {
     public:
         MediaSink(MFVideoRendererControl *rendererControl)
@@ -1478,6 +1518,14 @@ namespace
             m_stream->present();
         }
 
+        void clearScheduledFrame()
+        {
+            QMutexLocker locker(&m_mutex);
+            if (m_shutdown)
+                return;
+            m_stream->clearScheduledFrame();
+        }
+
         MFTIME getTime()
         {
             QMutexLocker locker(&m_mutex);
@@ -1501,10 +1549,14 @@ namespace
                 return E_POINTER;
             if (riid == IID_IMFMediaSink) {
                 *ppvObject = static_cast<IMFMediaSink*>(this);
+            } else if (riid == IID_IMFGetService) {
+                *ppvObject = static_cast<IMFGetService*>(this);
             } else if (riid == IID_IMFMediaSinkPreroll) {
                 *ppvObject = static_cast<IMFMediaSinkPreroll*>(this);
             } else if (riid == IID_IMFClockStateSink) {
                 *ppvObject = static_cast<IMFClockStateSink*>(this);
+            } else if (riid == IID_IMFRateSupport) {
+                *ppvObject = static_cast<IMFRateSupport*>(this);
             } else if (riid == IID_IUnknown) {
                 *ppvObject = static_cast<IUnknown*>(static_cast<IMFFinalizableMediaSink*>(this));
             } else {
@@ -1529,7 +1581,19 @@ namespace
             return cRef;
         }
 
+        // IMFGetService methods
+        STDMETHODIMP GetService(const GUID &guidService,
+                                const IID &riid,
+                                LPVOID *ppvObject)
+        {
+            if (!ppvObject)
+                return E_POINTER;
 
+            if (guidService != MF_RATE_CONTROL_SERVICE)
+                return MF_E_UNSUPPORTED_SERVICE;
+
+            return QueryInterface(riid, ppvObject);
+        }
 
         //IMFMediaSinkPreroll
         STDMETHODIMP NotifyPreroll(MFTIME hnsUpcomingStartTime)
@@ -1724,6 +1788,68 @@ namespace
             return m_stream->setRate(flRate);
         }
 
+        // IMFRateSupport methods
+        STDMETHODIMP GetFastestRate(MFRATE_DIRECTION eDirection,
+                                    BOOL fThin,
+                                    float *pflRate)
+        {
+            if (!pflRate)
+                return E_POINTER;
+
+            *pflRate = (fThin ? 8.f : 2.0f) * (eDirection == MFRATE_FORWARD ? 1 : -1) ;
+
+            return S_OK;
+        }
+
+        STDMETHODIMP GetSlowestRate(MFRATE_DIRECTION eDirection,
+                                    BOOL fThin,
+                                    float *pflRate)
+        {
+            Q_UNUSED(eDirection);
+            Q_UNUSED(fThin);
+
+            if (!pflRate)
+                return E_POINTER;
+
+            // we support any rate
+            *pflRate = 0.f;
+
+            return S_OK;
+        }
+
+        STDMETHODIMP IsRateSupported(BOOL fThin,
+                                     float flRate,
+                                     float *pflNearestSupportedRate)
+        {
+            HRESULT hr = S_OK;
+
+            if (!qFuzzyIsNull(flRate)) {
+                MFRATE_DIRECTION direction = flRate > 0.f ? MFRATE_FORWARD
+                                                          : MFRATE_REVERSE;
+
+                float fastestRate = 0.f;
+                float slowestRate = 0.f;
+                GetFastestRate(direction, fThin, &fastestRate);
+                GetSlowestRate(direction, fThin, &slowestRate);
+
+                if (direction == MFRATE_REVERSE)
+                    qSwap(fastestRate, slowestRate);
+
+                if (flRate < slowestRate || flRate > fastestRate) {
+                    hr = MF_E_UNSUPPORTED_RATE;
+                    if (pflNearestSupportedRate) {
+                        *pflNearestSupportedRate = qBound(slowestRate,
+                                                          flRate,
+                                                          fastestRate);
+                    }
+                }
+            } else if (pflNearestSupportedRate) {
+                *pflNearestSupportedRate = flRate;
+            }
+
+            return hr;
+        }
+
     private:
         long    m_cRef;
         QMutex  m_mutex;
@@ -2064,6 +2190,14 @@ namespace
             m_sink->present();
         }
 
+        void clearScheduledFrame()
+        {
+            QMutexLocker locker(&m_mutex);
+            if (!m_sink)
+                return;
+            m_sink->clearScheduledFrame();
+        }
+
         MFTIME getTime()
         {
             if (m_sink)
@@ -2168,10 +2302,16 @@ void MFVideoRendererControl::customEvent(QEvent *event)
         MFTIME targetTime = static_cast<MediaStream::PresentEvent*>(event)->targetTime();
         MFTIME currentTime = static_cast<VideoRendererActivate*>(m_currentActivate)->getTime();
         float playRate = static_cast<VideoRendererActivate*>(m_currentActivate)->getPlayRate();
-        if (playRate > 0.0001f && targetTime > currentTime)
-            QTimer::singleShot(int((float)((targetTime - currentTime) / 10000) / playRate), this, SLOT(present()));
-        else
+        if (!qFuzzyIsNull(playRate) && targetTime != currentTime) {
+            // If the scheduled frame is too late, skip it
+            const int interval = ((targetTime - currentTime) / 10000) / playRate;
+            if (interval < 0)
+                static_cast<VideoRendererActivate*>(m_currentActivate)->clearScheduledFrame();
+            else
+                QTimer::singleShot(interval, this, SLOT(present()));
+        } else {
             present();
+        }
         return;
     }
     if (event->type() >= MediaStream::StartSurface) {
